# coding=utf-8
from django import forms
from django_helpers.apps.form_renderer import FormRenderer
from models import File

__author__ = 'ajumell'


class AddFileForm(forms.ModelForm):
    class Meta:
        model = File

class EditFileForm(forms.ModelForm):
    class Meta:
        model = File
        exclude = ('image', )


class AddFileFormRenderer(FormRenderer):
    form_id = 'add-file-form'
    form_submit = 'Upload File'



# coding=utf-8
__author__ = 'ajumell'

from django_helpers.apps.supersized import SuperSized
from django_helpers.apps.supersized.models import SuperSizedSlider
from django import template
from django_helpers.helpers.templatetags import TemplateNode, parse_args


class SuperSizedNode(TemplateNode):
    def render(self, context):
        name = self.args[0].resolve(context)
        slider_obj = SuperSizedSlider.objects.get(name=name)
        slider = SuperSized()
        for attr in dir(slider_obj):
            if not attr.startswith('__'):
                if hasattr(slider, attr):
                    val = getattr(slider_obj, attr)
                    setattr(slider, attr, val)

        for img in slider_obj.slides():
            url = img.image.url
            try:
                thumb = img.image.thumbnail
            except Exception, dt:
                print 'Error :', dt
                thumb = url
            caption = img.caption
            link = img.link
            slider.add_slide(url, caption, thumb, link)
        return unicode(slider)


register = template.Library()


@register.tag
def super_sized(parser, token):
    bits = token.split_contents()
    args, kwargs = parse_args(bits, parser)
    return SuperSizedNode(args, kwargs)



# coding=utf-8
from django_helpers.apps.image_manager.fields import ImageManagerImageField
from django_helpers.apps.supersized import TransitionChoices
from django.db import models

__author__ = 'ajumell'

class SuperSizedSlider(models.Model):
    name = models.CharField(max_length=75, unique=True)
    autoplay = models.BooleanField(default=True)
    fit_always = models.BooleanField(default=False)
    fit_landscape = models.BooleanField(default=False)
    fit_portrait = models.BooleanField(default=True)
    horizontal_center = models.BooleanField(default=False)
    image_protect = models.BooleanField(default=True)
    keyboard_nav = models.BooleanField(default=True)
    min_height = models.IntegerField(default=0)
    min_width = models.IntegerField(default=0)
    new_window = models.BooleanField(default=True)
    pause_hover = models.BooleanField(default=False)
    performance = models.IntegerField(default=1)
    random = models.BooleanField(default=False)
    slideshow = models.BooleanField(default=True)
    slide_interval = models.IntegerField(default=5000)
    slide_links = models.BooleanField(default=False)
    start_slide = models.IntegerField(default=1)
    stop_loop = models.BooleanField(default=False)
    thumb_links = models.BooleanField(default=True)
    thumbnail_navigation = models.BooleanField(default=True)
    transition = models.CharField(default='fade', choices=TransitionChoices, max_length=25)
    transition_speed = models.IntegerField(default=750)
    vertical_center = models.BooleanField(default=True)
    progress_bar = models.BooleanField(default=True)
    mouse_scrub = models.BooleanField(default=True)
    show_thumbnails = models.BooleanField(default=False)
    hide_controls = models.BooleanField(default=False)


    def slides(self):
        return self.supersizedslide_set.all()


    def __unicode__(self):
        return self.name


class SuperSizedSlide(models.Model):
    slider = models.ForeignKey(SuperSizedSlider)
    image = ImageManagerImageField()
    caption = models.CharField(max_length=255, blank=True)
    link = models.URLField(blank=True)

    def __unicode__(self):
        return "%d - %s" % (self.id, self.image.name)



__author__ = 'ajumell'

from django.db import connection

try:
    import sqlparse
except ImportError:
    sqlparse = None


def query_count():
    return len(connection.queries)


def last_query():
    l = query_count() - 1
    query = connection.queries[l]['sql']
    if sqlparse is not None:
        query = sqlparse.format(query, reindent=True, keyword_case='upper')
    return query


__author__ = 'ajumell'


class DataTableFormFilter(object):
    form_id = None
    field_connection = {}

    def filter(self, query, DICT):
        fields = self.field_connection
        filter_dict = {}
        for field in fields:
            val = DICT.get(field, '')
            if val is not None:
                filter_dict[fields[field]] = val
        query = query.filter(**filter_dict)
        return query



# coding=utf-8
from json import loads

from django.shortcuts import render
from django.http import HttpResponseRedirect
from django.utils.crypto import get_random_string


__author__ = 'ajumell'

_no = '__no_js_data__'
_key = '__check_js_key__'
_name = '__check_js_name__'
_sess_name = "js_data"

# TODO: Make this in to a app.
# TODO: Add Modernizr data to request.
# TODO: Set data to session from request if session is flushed.


class CheckJsMiddleware:
    def process_request(self, request):
        """
        Provides Modernizr information in the django side of the app.
        @param request: The current http request object.
        @type request:django.http.HttpRequest
        @return: None
        """
        method = request.method
        session = request.session
        js_data = session.get(_sess_name, _no)
        if js_data == _no:
            if method == "POST":
                post = request.POST

                name = session.get(_name)
                key = session.get(_key)

                if name is None:
                    raise Exception('Name not found')

                if key is None:
                    raise Exception('Key not found')

                if name not in post:
                    raise Exception('Name "%s" is not in POST' % name)

                if key not in post:
                    raise Exception('Key "%s" is not in POST' % name)

                js_str = post.get(name, _no)
                if js_str == _no:
                    # Hack attempt
                    raise

                if js_str == name:
                    js_data = False
                else:
                    try:
                        js_data = loads(js_str)
                    except Exception:
                        raise Exception('%s != %s' % (name, js_str))
                session[_sess_name] = js_data
                return HttpResponseRedirect(request.get_full_path())
            else:
                key = session.get(_key) or get_random_string(12)
                name = session.get(_name) or get_random_string(12)

                session[_key] = key
                session[_name] = name

                return render(request, 'modernizr/detect-js.html', {
                    "key": key,
                    "name": name
                })
        else:
            request.Modernizr = js_data



# encoding: utf-8

"""
A filter to highlight code blocks in html with Pygments and BeautifulSoup.

    {% load highlight_code %}

    {{ var_with_code|highlight|safe }}
"""

import pygments
from pygments.formatters.html import HtmlFormatter
from pygments.lexers import get_lexer_by_name, get_all_lexers

from django import template
from django.template import Context

from django.template.defaultfilters import stringfilter
from django_helpers.helpers.templatetags import parse_verbatim

register = template.Library()


def _highlight_html(code, lexer='html+django'):
    try:
        code = code.lstrip('\n')
        code = code.rstrip()
        lexer = get_lexer_by_name(lexer)
        formatter = HtmlFormatter(cssclass='codehilite', linenos='inline')
        code_hl = pygments.highlight(code, lexer, formatter)
        return code_hl
    except:
        raise


@register.filter
@stringfilter
def highlight_var(html):
    return _highlight_html(html)


@register.simple_tag(takes_context=False)
def highlight_file(name, *args):
    fp = file(name)
    html = fp.read()
    fp.close()
    return _highlight_html(html)


class HighlightBlock(template.Node):
    def __init__(self, contents, code_format):
        self.contents = contents
        self.format = code_format

    def render(self, context):
        return _highlight_html(self.contents, self.format)

# noinspection PyUnusedLocal
@register.tag
def highlight(parser, token):
    bits = token.contents.split()
    code_format = bits[1] if len(bits) > 1 else 'html+django'
    contents = parse_verbatim(parser, 'endhighlight')
    return HighlightBlock(contents, code_format)


# coding=utf-8
from django.utils.safestring import mark_safe
from html import generate_code


def icon_text(icon, color, text=''):
    if icon == '' or icon is None:
        return ''

    if color == '' or color == 'black':
        color = 'black'
    else:
        color = 'white'

    return generate_code('i', {
        'class': (
            'icon',
            'icon-' + color,
            'icon-' + icon,
        )
    }, text)


def bootstrap_button_dict(text, color, link, icon='', size=''):
    return {
        'text': text,
        'color': color,
        'link': link,
        'icon': icon,
        'size': size
    }


def content(text, icon, color):
    return mark_safe(icon_text(icon, color) + text)


def button_classes(color='', size=''):
    arr = ['btn']
    if color != '' and color is not None:
        arr.append('btn-' + color)

    if size != '' and size is not None:
        arr.append('btn-' + size)
    return arr


def link_button(text, color='', link='#', icon='', size=''):
    return generate_code('a', {
        'class': button_classes(color, size),
        'href': link
    }, content(text, icon, color))


def button(text, color='', button_type='submit', icon='', size='', value=None):
    return generate_code('button', {
        'class': button_classes(color, size),
        'type': button_type,
        'value': value
    }, content(text, icon, color))


def link_button_set(buttons):
    html = generate_code('div', {
        'class': 'btn-group'
    }, start_only=True)
    for button in buttons:
        html += link_button(**button)
    html += '</div>'
    return html


"""
            form: [forms+'editable-form-bootstrap.js'],
            container: [containers+'editable-popover.js'],
            inputs: [
                inputs+'date/bootstrap-datepicker/js/bootstrap-datepicker.js',
                inputs+'date/date.js',
                inputs+'date/datefield.js',
                inputs+'datetime/datetime.js',
                inputs+'datetime/datetimefield.js',
                //don't build datetime lib, should be included manually
                //inputs+'datetime/bootstrap-datetimepicker/js/bootstrap-datetimepicker.js',
                inputs+'typeahead.js'
                ],
            css: [
                inputs+'date/bootstrap-datepicker/css/datepicker.css'
               //don't build datetime lib, should be included manually
               //inputs+'datetime/bootstrap-datetimepicker/css/datetimepicker.css'
                ]



                    var js = [
    '<banner:meta.banner>',
    forms+'editable-form.js',
    forms+'editable-form-utils.js',
    containers+'editable-container.js',
    containers+'editable-inline.js',
    lib+'element/editable-element.js',
    inputs+'abstract.js',
    inputs+'list.js',
    inputs+'text.js',
    inputs+'textarea.js',
    inputs+'select.js',
    inputs+'checklist.js',
    inputs+'html5types.js',
    inputs+'select2/select2.js',
    inputs+'combodate/lib/combodate.js',
    inputs+'combodate/combodate.js'
    ]
        var css = [
    '<banner:meta.banner>',
    forms+'editable-form.css',
    containers+'editable-container.css',
    lib+'element/editable-element.css'
    ];


    lib = 'src/'
    forms = lib+'editable-form/'
    inputs = lib+'inputs/'
    containers = lib+'containers/'
"""

from django_helpers.apps.static_manager import jQueryURL, BootstrapURL, AjaxCsrfURL

_lib = 'editable/'
_forms_dir = _lib + 'editable-form/'
_inputs_dir = _lib + 'inputs/'
_containers_dir = _lib + 'containers/'


# noinspection PyUnusedLocal
def _require(css, *js):
    arr = []
    for folder in js:
        for f in folder:
            arr.append(f)
        #
    # if css is not None:
    #     for f in css:
    #         add_css_file(f)

    return arr


def require():
    form = [
        jQueryURL,
        BootstrapURL,
        AjaxCsrfURL,
        _forms_dir + 'editable-form-bootstrap.js'
    ]

    container = [_containers_dir + 'editable-popover.js']

    input_files = [
    ]

    css = [
        _forms_dir + 'editable-form.css',
        _containers_dir + 'editable-container.css',
        _lib + 'element/editable-element.css'
    ]

    js = [
        _forms_dir + 'editable-form.js',
        _forms_dir + 'editable-form-utils.js',
        _containers_dir + 'editable-container.js',
        _containers_dir + 'editable-inline.js',
        _lib + 'element/editable-element.js',
        _inputs_dir + 'abstract.js',
        _inputs_dir + 'list.js',
        _inputs_dir + 'text.js',
        _inputs_dir + 'textarea.js',
        _inputs_dir + 'select.js',
        _inputs_dir + 'checklist.js',
        _inputs_dir + 'html5types.js',
        _inputs_dir + 'select2/select2.js',
        _inputs_dir + 'combodate/lib/combodate.js',
        _inputs_dir + 'combodate/combodate.js'
    ]

    extras = [
        _lib + 'django.xeditable.js'
    ]

    return _require(css, js, form, container, input_files, extras)


def require_date_filed():
    js = [
        jQueryURL,
        BootstrapURL,
        'js/bootstrap-datepicker.js',
        _inputs_dir + 'date/date.js',
        _inputs_dir + 'date/datefield.js',
    ]
    _require(None, js)



# coding=utf-8
from django.template import TemplateSyntaxError, Node
from django.template.defaulttags import register
from django_helpers.helpers.templatetags import parse_args
from django_helpers.apps.data_tables.views import get_table

__author__ = 'ajumell'


class DataTableRendererNode(Node):
    def __init__(self, table_name, args=None, kwargs=None):
        self.table_name = table_name
        self.args = args
        self.kwargs = kwargs

    def render(self, context):
        table = get_table(self.table_name)
        return table.safe_string(kwargs=self.kwargs, args=self.args)


@register.tag
def render_data_table(parser, token):
    bits = token.split_contents()
    if len(bits) < 2:
        raise TemplateSyntaxError("'%s' needs at least two arguments." % bits[0])
    args, kwargs = parse_args(bits, parser)

    table_name = args[0]
    args = args[1:]
    return DataTableRendererNode(table_name, args, kwargs)


# coding=utf-8
from django.contrib.humanize.templatetags.humanize import naturaltime, ordinal, intcomma, naturalday
from . import DataTableColumn
from django.utils.html import linebreaks


class NaturalTimeColumn(DataTableColumn):
    def get_html(self, request, instance):
        value = DataTableColumn.get_html(self, request, instance)
        return naturaltime(value)


class NaturalDayColumn(DataTableColumn):
    def get_html(self, request, instance):
        value = DataTableColumn.get_html(self, request, instance)
        return naturalday(value)


class OrdinalIntegerColumn(DataTableColumn):
    def get_html(self, request, instance):
        value = DataTableColumn.get_html(self, request, instance)
        return ordinal(value)


class LineBreaksColumn(DataTableColumn):
    def get_html(self, request, instance):
        value = DataTableColumn.get_html(self, request, instance)
        return linebreaks(value)


class ComaSeparatedIntegerColumn(DataTableColumn):
    def get_html(self, request, instance):
        value = DataTableColumn.get_html(self, request, instance)
        return intcomma(value)


__author__ = 'ajumell'


class StaticFileNotFoundException(Exception):
    def __init__(self, name):
        Exception.__init__(self, 'Static File "%s" not found.' % name)


class NotInProduction(Exception):
    def __init__(self):
        Exception.__init__(self, 'Static manager is not intended to use in production.')


class SyntaxErrorException(Exception):
    def __init__(self, obj):
        msg = 'Render JS Should only pass pure Javascript.'
        msg += '\nJavascript Syntax Error occured in object. %s' % str(type(obj))
        Exception.__init__(self, msg)


class PYESPRIMAException(Exception):
    def __init__(self):
        msg = "pyesprima is required for checking the syntax of javascript."
        msg += '\n Install it with pip install pyesprima.'

        Exception.__init__(self, msg)


class AppNameMissingException(Exception):
    def __init__(self):
        msg = "app_name is missing. It should be passed along with template tag."
        msg += " or the context should have a data with app_name."

        Exception.__init__(self, msg)


import os
from json import load, dump

from django.utils.importlib import import_module

from django_helpers import get_settings_val

_IS_DEBUG = get_settings_val('DEBUG', True)
_STORAGES = None
_STATIC_FILES_STORAGE_FILE_NAME = get_settings_val('STATIC_FILES_STORAGE_FILE_NAME', 'STATIC_FILES_STORAGE')


def _storage_file_name():
    """
    Calculates the file path for the storage of the static files.
    @return: The absolute file name for the storage file name.
    """
    settings = os.environ.get('DJANGO_SETTINGS_MODULE')
    module = import_module(settings)
    root = os.path.dirname(module.__file__)
    root = os.path.abspath(root)
    return os.path.join(root, _STATIC_FILES_STORAGE_FILE_NAME)


def _load_data():
    """
    @summary Loads the data from the file.
    """
    global _STORAGES
    path = _storage_file_name()
    try:
        fp = file(path)
        _STORAGES = load(fp)
        fp.close()
    except Exception:
        _STORAGES = {}


def _storage(file_type, app_name):
    if _STORAGES is None:
        _load_data()

    file_storage = _STORAGES.get(file_type)
    if file_storage is None:
        file_storage = {}
        _STORAGES[file_type] = file_storage

    app_storage = file_storage.get(app_name)
    if app_storage is None:
        app_storage = []
        file_storage[app_name] = app_storage

    return app_storage


def save_data():
    """
    @summary Saves the data to the file.
    """
    path = _storage_file_name()
    fp = file(path, 'w')
    dump(_STORAGES, fp, indent=4, sort_keys=True)
    fp.close()


def add_file(file_type, app_name, path, cdn):
    if not _IS_DEBUG:
        return
    storage = _storage(file_type, app_name)

    if cdn is not None:
        path = (path, cdn)

    if path not in storage:
        storage.append(path)


def add_files(file_type, app_name, paths):
    if not _IS_DEBUG:
        return
    storage = _storage(file_type, app_name)
    for path in paths:
        if path not in storage:
            storage.append(path)


def add_js(app_name, path, cdn):
    add_file('js', app_name, path, cdn)


def add_js_files(app_name, files):
    add_files('js', app_name, files)
    save_data()


def add_css(app_name, path, cdn=None):
    add_file('css', app_name, path, cdn)


def get_files(file_type, app_name):
    if not _IS_DEBUG:
        raise

    return_arr = []
    storage = _storage(file_type, app_name)
    for file_name in storage:
        if type(file_name) is tuple:
            file_name, cdn = file_name
        return_arr.append(file_name)
    return return_arr



# coding=utf-8
from django.forms import widgets

from django_helpers.helpers.views import render_to_string
from django_helpers.apps.static_manager import jQueryURL
from django_helpers.apps.forms.widgets import Widget


__author__ = 'ajumell'


class TinyMCEEditorOptions(object):
    pass


class TinyMCEWidget(Widget, widgets.Textarea):
    def __init__(self, *args, **kwargs):
        self.js_files = [jQueryURL, 'tiny_mce/tiny_mce.js', 'tiny_mce/jquery.tinymce.js']
        widgets.Textarea.__init__(self, *args, **kwargs)

    def render_js(self):
        return render_to_string('django-helpers/forms/tiny_mce.js', {
        })


from types import StringTypes
from django.core.urlresolvers import reverse, resolve
from django.template import RequestContext, Parser, resolve_variable
from django.utils.safestring import mark_safe

from django_helpers.helpers.views import render_to_string
from django_helpers.apps.static_manager import jQueryURL

from exceptions import InvalidContextException, MenuIDNotFoundException, MultipleMenuWithSameIDException

from helpers import *

__author__ = 'ajumell'

__all__ = ('Menu', 'register', 'has_menu', 'get_menu', 'menu_item')

_REGISTERED_MENU = {}


class Menu(object):
    menu_classes = []
    items = []
    has_js = True
    menu_id = None
    highlight_current = True
    ajax = False

    def __init__(self, context, items=None):

        if not isinstance(context, RequestContext):
            InvalidContextException()

        if self.menu_id is None:
            MenuIDNotFoundException()

        request = resolve_variable('request', context)
        self.current_url = resolve(request.path_info).url_name
        self.path = request.get_full_path()
        self.current = False
        self.current_level = None

        self.items = self.items[:]

        if items is not None and self.items is None:
            self.items = items

        self.context = context
        self.parser = Parser('')
        self.user = resolve_variable('user', context)

        self.js_files = [jQueryURL, 'js/superfish.min.js', 'js/jquery.django.ajax.js']

    def format_text(self, text, params):
        context = self.context
        args = []
        for param in params:
            default = None
            if type(param) in (tuple, list):
                param, default = param
            try:
                op = resolve_variable(param, context)
            except:
                op = default if default is not None else param
            args.append(op)
        return text % tuple(args)

    def should_render(self, item):
        user = self.user
        perms = item.get('perms')
        need_auth = item.get('need_auth')
        need_guest = item.get('need_guest')

        if need_auth and not user.is_authenticated():
            return False

        if need_guest and not user.is_anonymous():
            return False

        has_perm = True
        if perms is not None:
            for perm in perms:
                has_perm = has_perm and user.has_perm(perm)
                if not has_perm:
                    return False
        return has_perm

    def get_template(self, level, has_sub_menu, decedent_current, is_current):
        return 'menu/simple-item.html'

    def extra_item_context_vars(self, item, level, d):
        pass

    def can_highlight(self, item, current_link, no_reverse, decedent_current):
        pass

    def render_item(self, item, level):
        text = item.get('text')
        link_name = item.get('link')

        text_params = item.get('text_params')
        if text_params is not None:
            text = self.format_text(text, text_params)

        no_reverse = item.get('no_reverse', False)

        if not no_reverse:
            link = reverse(link_name, args=item.get('link_args'), kwargs=item.get('link_kwargs'))
        else:
            link = link_name

        if link == self.path:
            self.current = True
            self.current_level = level

        sub_menu_html = ''
        sub_menu_items = item.get('sub_menu', None)
        has_sub_menu = False

        if sub_menu_items is not None:

            if hasattr(sub_menu_items, '__call__'):
                sub_menu_items = sub_menu_items()

            sub_menu = self.render_list(sub_menu_items, level + 1)
            has_sub_menu = sub_menu is not ''
            if has_sub_menu:
                sub_menu_html += self.render_menu_start(level + 1)
                sub_menu_html += sub_menu
                sub_menu_html += self.render_menu_end(level + 1)

        is_current = self.current and self.current_level == level and link == self.path
        decedent_current = self.current and self.current_level > level

        # Can Highlight
        highlighted = is_current or decedent_current
        extra_highlight_options = item.get('highlight-names', None)
        if type(extra_highlight_options) in StringTypes:
            extra_highlight_options = [extra_highlight_options]

        if type(extra_highlight_options) in (tuple, list):
            highlighted = highlighted or self.current_url in extra_highlight_options

        d = {
            'url': link,
            'text': text,
            'is_current': is_current,
            'decedent_current': decedent_current,
            'can_highlight': highlighted,
            'has_sub_menu': has_sub_menu,
            'sub_menu': mark_safe(sub_menu_html)
        }

        self.extra_item_context_vars(item, level, d)

        if level == 0:
            self.current = False
            self.current_level = None

        return render_to_string(self.get_template(level, has_sub_menu, decedent_current, is_current), d)

    def render_list(self, items, level):
        op = []
        for item in items:
            if self.should_render(item):
                op.append(self.render_item(item, level))
        return '\n'.join(op)

    def render_menu_start(self, level):
        if level is 0:
            classes = self.menu_classes[:]
            return '<ul class="sf-menu %s" id="%s">' % (' '.join(classes), self.menu_id)
        else:
            return '<ul>'

    # noinspection PyUnusedLocal
    def render_menu_end(self, level):
        return '</ul>'

    def render(self):
        op = ''

        op += self.render_menu_start(0)
        op += self.render_list(self.items, 0)
        op += self.render_menu_end(0)

        return op

    def render_js(self):
        op = render_to_string('menu/super-fish.js', {
            'menu_id': self.menu_id,
            'ajax': self.ajax
        })
        return op


def register(cls):
    menu_id = getattr(cls, 'menu_id', None)

    if menu_id is None:
        raise MenuIDNotFoundException()

    if menu_id in _REGISTERED_MENU:
        if _REGISTERED_MENU[menu_id] != cls:
            raise MultipleMenuWithSameIDException()
    else:
        _REGISTERED_MENU[menu_id] = cls
    return cls


def has_menu(name):
    return name in _REGISTERED_MENU


def get_menu(name):
    return _REGISTERED_MENU[name]


# coding=utf-8
import os.path


def load_from_search_strings_file():
    f = None
    try:
        f = open(os.path.join(os.path.dirname(__file__), 'search_strings.txt'))
        ss = f.readlines()
    finally:
        if f:
            f.close()
    return [s.strip() for s in ss if not s.startswith('#')]


search_strings = load_from_search_strings_file()


def _menu_item(text, link, text_params=None, perms=None, need_auth=None, link_args=None, link_kwargs=None, need_guest=None, no_reverse=False, sub_menu=None, highlights=None):
    d = {
        'text': text,
        'link': link,
        'text_params': text_params,
        'need_auth': need_auth,
        'need_guest': need_guest,
        'perms': perms,

        'link_args': link_args,
        'link_kwargs': link_kwargs,
        'no_reverse': no_reverse,
        'highlight-names': highlights
    }

    if sub_menu is not None:
        d['sub_menu'] = sub_menu

    return d


def menu_item(text, link, text_params=None, perms=None, need_auth=None, link_args=None, link_kwargs=None, need_guest=None, no_reverse=False, sub_menu=None, **kwargs):
    return _menu_item(text, link, text_params=text_params, perms=perms, need_auth=need_auth, link_args=link_args, link_kwargs=link_kwargs, need_guest=need_guest, no_reverse=no_reverse,
                      sub_menu=sub_menu, **kwargs)


def user_menu_item(text, link, text_params=None, perms=None, link_args=None, link_kwargs=None, no_reverse=False, sub_menu=None, **kwargs):
    return _menu_item(text, link, text_params=text_params, perms=perms, need_auth=True, link_args=link_args, link_kwargs=link_kwargs, need_guest=False, no_reverse=no_reverse,
                      sub_menu=sub_menu, **kwargs)


def guest_item(text, link, text_params=None, link_args=None, link_kwargs=None, no_reverse=False, sub_menu=None, **kwargs):
    return _menu_item(text, link, text_params=text_params, link_args=link_args, link_kwargs=link_kwargs, need_guest=True, no_reverse=no_reverse, sub_menu=sub_menu, **kwargs)


#
#   Sub Menu Items
#
def item_with_sub_menu(text, text_params=None, link="#", perms=None, need_auth=None, need_guest=None, sub_menu=None, **kwargs):
    return _menu_item(text, text_params=text_params, link=link, perms=perms, need_auth=need_auth, need_guest=need_guest, sub_menu=sub_menu, **kwargs)


def guest_item_with_sub_menu(text, text_params=None, link="#", sub_menu=None, **kwargs):
    return _menu_item(text, text_params=text_params, link=link, need_auth=False, need_guest=True, sub_menu=sub_menu, **kwargs)


def user_item_with_sub_menu(text, text_params=None, link="#", perms=None, sub_menu=None, **kwargs):
    return _menu_item(text, text_params=text_params, link=link, perms=perms, need_auth=True, need_guest=False, sub_menu=sub_menu, **kwargs)



# coding=utf-8
__author__ = 'ajumell'

from django.contrib.admin import site, ModelAdmin
from models import NivoSlider, NivoSliderSlide

class NivoSliderAdmin(ModelAdmin):
    pass

site.register(NivoSlider)
site.register(NivoSliderSlide)




# coding=utf-8
from django.core.files.storage import get_storage_class
from django_helpers import get_settings_val, check_settings_val_exist

__author__ = 'ajumell'

from django.db import models

check_settings_val_exist('MEDIA_ROOT')
check_settings_val_exist('MEDIA_URL')

STORAGE = get_storage_class(get_settings_val('FILE_MANAGER_STORAGE', None))
UPLOAD_TO = get_settings_val('FILE_MANAGER_FOLDER', 'image-manager/')


# noinspection PyShadowingBuiltins
class File(models.Model):
    name = models.SlugField(max_length=100, unique=True)
    small_description = models.CharField(max_length=255, blank=True)
    large_description = models.TextField(blank=True)
    file = models.FileField(upload_to=UPLOAD_TO, storage=STORAGE())
    folder = models.CharField(max_length=255, blank=True)

    def __unicode__(self):
        return self.name

    @property
    def url(self):
        return self.file.url


class FileGroup(models.Model):
    name = models.SlugField(max_length=255)
    files = models.ManyToManyField(File)

    def __unicode__(self):
        return self.name


# coding=utf-8
from django.core.urlresolvers import reverse
from django.template.defaultfilters import date
from django.utils.dateparse import parse_datetime, parse_date
from columns import DataTableColumn, get_value
from django_helpers.utils.bootstrap import link_button, generate_code, link_button_set


class BootstrapButtonColumn(DataTableColumn):
    def __init__(self, text, color, link, icon='', size=None, title_class=None, title_width="85px", title='', field='id'):
        DataTableColumn.__init__(self, field, title, False, False, title_class, title_width=title_width)
        self.text = text
        self.color = color
        self.link = link
        self.icon = icon
        self.size = size

    def get_html(self, request, instance):
        value = get_value(instance, self.field)
        try:
            link = reverse(self.link, args=(
                value,
            ))
        except:
            link = self.link

        return link_button(self.text, self.color, link, self.icon, self.size)


class BootstrapButtonSetColumn(DataTableColumn):
    def __init__(self, buttons, field='id', title='', title_class=None, title_width="auto", ):
        DataTableColumn.__init__(self, field, title, False, False, title_class, title_width)
        self.buttons = buttons

    def get_html(self, request, instance):
        data = []
        for button in self.buttons:
            text = button.get('text', '')
            color = button.get('color', '')
            link = button.get('link', '')
            icon = button.get('icon', '')
            size = button.get('size', '')

            field = button.get('field', self.field)
            value = getattr(instance, field)
            try:
                link = reverse(link, args=(value, ))
            except:
                pass
            data.append({
                'link': link,
                'text': text,
                'color': color,
                'icon': icon,
                'size': size
            })

        return link_button_set(data)


class BootstrapInfoButton(BootstrapButtonColumn):
    def __init__(self, text, link, icon='', size=None, title_class=None, title_width="85px", title='', field='id'):
        BootstrapButtonColumn.__init__(text, 'info', link, icon, size, title_class, title_width, title, field)


class BootstrapSuccessButton(BootstrapButtonColumn):
    def __init__(self, text, link, icon='', size=None, title_class=None, title_width="85px", title='', field='id'):
        BootstrapButtonColumn.__init__(text, 'success', link, icon, size, title_class, title_width, title, field)


class BootstrapErrorButton(BootstrapButtonColumn):
    def __init__(self, text, link, icon='', size=None, title_class=None, title_width="85px", title='', field='id'):
        BootstrapButtonColumn.__init__(text, 'error', link, icon, size, title_class, title_width, title, field)


class BootstrapWarningButton(BootstrapButtonColumn):
    def __init__(self, text, link, icon='', size=None, title_class=None, title_width="85px", title='', field='id'):
        BootstrapButtonColumn.__init__(text, 'warning', link, icon, size, title_class, title_width, title, field)


class BootstrapPrimaryButton(BootstrapButtonColumn):
    def __init__(self, text, link, icon='', size=None, title_class=None, title_width="85px", title='', field='id'):
        BootstrapButtonColumn.__init__(text, 'primary', link, icon, size, title_class, title_width, title, field)


class StaticLinkDataTableColumn(DataTableColumn):
    def __init__(self, text, link, title_width="20px", title='', field='id', title_class=None, search_lookup='contains'):
        DataTableColumn.__init__(self, field, title, False, False, title_class, title_width, search_lookup=search_lookup)
        self.text = text
        self.link = link

    def get_html(self, request, instance):
        value = getattr(instance, self.field)
        try:
            link = reverse(self.link, args=(
                value,
            ))
        except:
            link = self.link

        return generate_code('a', {
            'href': link
        }, self.text)


class LinkDataTableColumn(DataTableColumn):
    def __init__(self, field, link, title, searchable, sortable, link_field=None, title_class=None, title_width=None, search_lookup='contains', format_expression=None):
        DataTableColumn.__init__(self, field, title, searchable, sortable, title_class, title_width, search_lookup=search_lookup, format_expression=format_expression)
        self.link = link
        self.link_field = link_field

    def get_fields(self):
        field = self.field
        link_field = self.link_field
        if field == link_field:
            return field
        return field, link_field

    def get_html(self, request, instance):
        value = get_value(instance, self.field, self.format_expression)
        link_arg = get_value(instance, self.link_field or self.field)

        try:
            link = reverse(self.link, args=(
                link_arg,
            ))
        except:
            link = self.link

        return generate_code('a', {
            'href': link
        }, value)


class EmailDataTableColumn(DataTableColumn):
    def __init__(self, field, title, searchable, sortable, title_class=None, title_width=None,
                 annotate=None, search_lookup='contains', format_expression=None):
        DataTableColumn.__init__(self, field, title, searchable, sortable, title_class, title_width, False, annotate, search_lookup, format_expression)

    def get_html(self, request, instance):
        value = DataTableColumn.get_html(self, request, instance)
        return generate_code('a', {
            'href': 'mailto:' + value
        }, value)


class DateDataTableColumn(DataTableColumn):
    def __init__(self, field, title, searchable, sortable, date_format=None, title_class=None, title_width=None,
                 editable=False, annotate=None, search_lookup='contains', format_expression=None):
        DataTableColumn.__init__(self, field, title, searchable, sortable, title_class, title_width, editable, annotate, search_lookup, format_expression)
        self.date_format = date_format

    def get_html(self, request, instance):
        value = get_value(instance, self.field)
        if self.annotate is not None:
            str_value = value
            value = parse_date(str_value)
            if value is None:
                value = parse_datetime(str_value)
        value = date(value, self.date_format)
        if self.format_expression is not None:
            value = self.format_expression % value
        return value


class CheckboxColoum(DataTableColumn):
    def __init__(self, title_class=None, name=None, extra_classes=None, title_width="25px", title='', field='id'):
        DataTableColumn.__init__(self, field, title, False, False, title_class, title_width)
        # self.title = mark_safe('<input type="checkbox">')
        self.name = name
        self.extra_classes = extra_classes

    def get_html(self, request, instance):
        attrs = {
            'type': 'checkbox',
            'class': self.extra_classes,
            'name': self.name,
            'value': get_value(instance, self.field)
        }
        return generate_code('input', attrs, self_closing=True)


class ImageField(DataTableColumn):
    def __init__(self, field, title='', convert_url=True, title_class=None, extra_classes=None, title_width="25px"):
        DataTableColumn.__init__(self, field, title, False, False, title_class, title_width)
        self.convert_url = convert_url
        self.extra_classes = extra_classes

    def get_html(self, request, instance):
        obj = get_value(instance, self.field)
        try:
            url = obj.thumbnail
        except:
            url = obj.url
        attrs = {
            'class': self.extra_classes,
            'src': url
        }
        return generate_code('img', attrs, self_closing=True)


class BooleanImageField(DataTableColumn):
    def __init__(self, field, yes_image, no_image, title='', title_class=None, extra_classes=None, title_width="25px"):
        DataTableColumn.__init__(self, field, title, False, False, title_class, title_width)
        self.extra_classes = extra_classes
        self.yes_image = yes_image
        self.no_image = no_image

    def get_html(self, request, instance):
        obj = get_value(instance, self.field)
        url = self.yes_image if obj else self.no_image
        attrs = {
            'class': self.extra_classes,
            'src': url
        }
        return generate_code('img', attrs, self_closing=True)



# coding=utf-8
from json import dumps
from hashlib import md5

from django.core.urlresolvers import reverse
from django.core.validators import *
from django.forms import *
from django.utils.html import escapejs
from django.utils.safestring import mark_safe

from django_helpers.helpers.views import render_json
from django_helpers.apps.static_manager import jQueryURL


__all__ = (
    'validation_preprocessor',
    'register'
)

SPRINTF_PARAMS = 'sprintf_params'
SPRINTF_VALUE = 'sprintf_value'
SPRINTF_LENGTH = 'sprintf_length'


def params(obj):
    """
    Returns the safe string json of the given object.
    @param obj:
    @return:
    """
    return mark_safe(dumps(obj))


def generate_rule(obj, msg):
    """
    Returns a dictionary with the given message and parameter.
    @param obj:
    @param msg:
    @return:
    """
    return {
        'params': params(obj),
        'msg': msg
    }


def wrap_str(msg, function):
    return mark_safe(u'$.dj.msgs.' + function + u'(' + unicode(msg) + u')')


def wrap_message(rule, function):
    msg = rule['msg']
    rule['msg'] = wrap_str(msg, function)


def err_msg(msg, function=None):
    msg = '"' + unicode(escapejs(msg)) + '"'
    if function is not None:
        msg = wrap_str(msg, function)
    else:
        msg = mark_safe(msg)
    return msg


def msg(validator, error_messages, function=None):
    error_message = validator.message or error_messages.get(validator.code)
    return err_msg(error_message, function)


def validation_preprocessor(renderer):
    """
    @type renderer: django_helpers.apps.form_renderer.FormRenderer
    @param renderer:
    """

    js_files = [
        jQueryURL,
        'form-renderer/js/jquery.validate.js',
        'form-renderer/js/sprintf.min.js',
        'form-renderer/js/jquery.validate.django.js'
    ]

    renderer.add_js_requirement(js_files)

    form = renderer.form
    for name, field in form.fields.items():
        validations = getattr(field, 'validations', None) or {}
        error_messages = field.error_messages

        if field.required:
            validations['required'] = generate_rule(True, err_msg(error_messages['required']))

        if isinstance(field, DateField):
            validations['date'] = generate_rule(True, err_msg(error_messages['invalid']))

        if isinstance(field, IntegerField):
            validations['integer'] = generate_rule(True, err_msg(error_messages['invalid']))

        if isinstance(field, DecimalField):
            validations['number'] = generate_rule(True, err_msg(error_messages['invalid']))

            digits = field.max_digits
            decimals = field.decimal_places

            if digits is not None:
                m = err_msg(error_messages['max_digits'], SPRINTF_PARAMS)
                validations['max_digits'] = generate_rule(digits, m)

            if decimals is not None:
                m = err_msg(error_messages['max_decimal_places'], SPRINTF_PARAMS)
                validations['max_decimal_places'] = generate_rule(decimals, m)

            if decimals is not None and digits is not None:
                whole_nums = digits - decimals
                m = err_msg(error_messages['max_whole_digits'], SPRINTF_PARAMS)
                validations['max_whole_digits'] = generate_rule(whole_nums, m)

        if isinstance(field, FloatField):
            validations['number'] = generate_rule(True, err_msg(error_messages['invalid']))

        if hasattr(form, 'clean_%s' % name):
            dt = {
                'url': reverse('validate-form'),
                'type': 'get',
                'data': {
                    'form_id': register_form(form),
                    'field_name': name
                }
            }
            validations['remote'] = generate_rule(dt, '')

        for validator in field.validators:

            obj = getattr(validator, 'limit_value', True)
            error_message = msg(validator, error_messages)
            rule = generate_rule(obj, error_message)

            if isinstance(validator, EmailValidator):
                wrap_message(rule, SPRINTF_VALUE)
                validations['email'] = rule

            elif isinstance(validator, URLValidator):
                validations['url'] = rule

            elif isinstance(validator, MinLengthValidator):
                wrap_message(rule, SPRINTF_LENGTH)
                validations['minlength'] = rule

            elif isinstance(validator, MaxLengthValidator):
                wrap_message(rule, SPRINTF_LENGTH)
                validations['maxlength'] = rule

            elif isinstance(validator, MinValueValidator):
                wrap_message(rule, SPRINTF_VALUE)
                validations['min'] = rule

            elif isinstance(validator, MaxValueValidator):
                wrap_message(rule, SPRINTF_VALUE)
                validations['max'] = rule

            elif isinstance(validator, RegexValidator):
                validations['regex'] = generate_rule(validator.regex.pattern, error_message)
                continue

        setattr(field, 'validations', validations)
    template = getattr(renderer, 'validation_template', 'form-renderer/preprocessors/validations.js')
    renderer.js_templates.append(template)


def remote_validation_preprocessor(renderer):
    form = renderer.form
    datas = getattr(renderer, 'remote_validations', None)
    if type(datas) is not dict:
        return
    for name, data in datas.items():
        field = form.fields[name]
        validations = getattr(field, 'validations', None) or {}
        if 'remote' in validations:
            raise Exception('Remote validation already exists.')
        if type(data) in (list, tuple):
            url = reverse(*data)
        elif type(data) is dict:
            url = reverse(**data)
        else:
            url = reverse(data)
        validations['remote'] = {
            'params': url
        }
        setattr(field, 'validations', validations)


#
#   Custom Validation
#
lookup = {}

_register_key = '__custom_validation_id__'


def register(cls):
    form_id = getattr(cls, _register_key, None)

    if form_id is None:
        form_id = md5(unicode(cls)).hexdigest()
        setattr(cls, _register_key, form_id)

    if form_id not in lookup:
        lookup[form_id] = cls

    return cls


def register_form(form):
    cls = type(form)
    form_id = getattr(cls, _register_key, None)
    if form_id is None:
        raise Exception("Form not registered for custom validation")

    if form_id not in lookup:
        raise Exception("Form not registered for custom validation")

    return form_id


def get_form(form_id, post_data):
    cls = lookup.get(form_id)
    if cls is None:
        return None
    return cls(post_data)


def validate_form(request):
    post = request.GET
    form_id = post.get('form_id')
    field_name = post.get('field_name')
    form = get_form(form_id, post)
    if form is None:
        return render_json("Server error.")

    errors = form.errors
    if field_name in errors:
        return render_json('<br />'.join(errors[field_name]))
    return render_json(True)



# coding=utf-8
__author__ = 'ajumell'

from django.db.models.fields.files import ImageFieldFile
from lib.thumbnail import ImageWithThumbsFieldFile, ImageWithThumbsField

try:
    from django_helpers.storages.flickr import FlickrStorage
except:
    FlickrStorage = None


class ThumbnailImageFieldFile(ImageWithThumbsFieldFile):
    def _get_thumbnail(self):
        storage = self.storage
        if hasattr(storage, 'thumbnail'):
            return self.storage.thumbnail(self.name)
        sizes = self.field.sizes
        w, h = sizes[0]
        return getattr(self, 'url_%sx%s' % (w, h))

    thumbnail = property(_get_thumbnail)

    def save(self, name, content, save=True):
        storage = self.storage
        # In Flickr storage there is no need for making thumbnails
        # manually. It will be done automatically.
        if FlickrStorage is not None and isinstance(storage, FlickrStorage):
            ImageFieldFile.save(self, name, content, save)
        else:
            ImageWithThumbsFieldFile.save(self, name, content, save)


class ThumbnailImageField(ImageWithThumbsField):
    attr_class = ThumbnailImageFieldFile

    def __init__(self, verbose_name=None, name=None, width_field=None, height_field=None, sizes=None, **kwargs):
        if sizes is None:
            sizes = ((150, 150), )
        ImageWithThumbsField.__init__(self, verbose_name, name, width_field, height_field, sizes, **kwargs)




from django.db.models import Q


class AutoComplete(object):
    query_set = None
    search_fields = []
    url_search_parameters = None
    requires_auth = False
    id_name = 'id'
    label_name = 'label'

    # noinspection PyUnusedLocal
    def get_queryset(self, request):
        """
        This function can be overridden to return custom query set.
        @return: The query set for performing extra searches.
        """
        return self.query_set

    def search_term(self, queryset, term):
        """
        Performs a simple database search for the term given.
        This function will performs a __contains in the given
        fields of the query set.

        @param term: str A search term
        @return: django.db.models.query.QuerySet a result query set.
        """
        query = None
        fields = self.search_fields
        for field in fields:
            kwargs = {field + "__contains": term}
            if query is None:
                query = Q(**kwargs)
            else:
                query = query | Q(**kwargs)
        return queryset.filter(query)

    def search_url_paramters(self, query, url_params):
        """
        This function performs a query lookup with the parameters from the url.
        @param query:
        @param url_params:
        @return:
        """
        url_search_parameters = self.url_search_parameters
        if url_search_parameters is not None:
            for parameter in url_search_parameters:
                args_dict = {parameter: url_params.get(parameter, '')}
                query = query.filter(**args_dict)
        return query

    def search(self, queryset, term, url_params):
        query = self.search_term(queryset, term)
        return self.search_url_paramters(query, url_params)

    def get_instances(self, ids):
        """
        Returns a queryset with the instances with given list of ids.
        This function is mainly used in many to many fields.

        @param ids: A list/enumerable of primary keys for which the datas has to be returned.
        @return: django.db.models.query.QuerySet
        """
        return self.query_set.filter(pk__in=ids)

    def get_instance(self, pk):
        """

        @param pk: The primary key for which the instance has to be returned.
        @return: A single instance of the current Model.
        """
        return self.query_set.get(pk=pk)

    def simple_format(self, instance, string, *fields):
        lst = []
        for term in fields:
            lst.append(getattr(instance, term))
        return string % tuple(lst)

    def format_value(self, instance):
        """
        Returns the formatted values that represents the current instance.
        @param instance:
        @return:
        """
        return unicode(instance)

    def get_search_term(self, request):
        """
        @author Muhammed K K
        @param request: django.http.HttpRequest current django request
        @return: str The term to be searched.
        """
        term = request.GET.get("term")
        if term is None:
            term = request.GET.get("q")
        return term

    def get_results(self, request, params):
        term = self.get_search_term(request)
        query = self.get_queryset(request)
        query = self.search(query, term, params)
        results = []
        for result in query:
            val = self.format_value(result)
            results.append({
                self.id_name: result.id,
                self.label_name: val,
            })
        return results


# coding=utf-8
import collections
from django.conf.urls import include, url
from urls import urlpatterns


def add_app_url(name, urls):
    reg_exp = r'^%s/' % name
    name += '-urls'

    for pattern in urlpatterns:
        if pattern._regex == reg_exp:
            return

    pattern = url(reg_exp, include(urls), name=name)
    urlpatterns.append(pattern)


def get_settings_val(name, default=None):
    from django.conf import settings

    obj = getattr(settings, name, default)
    return obj


def check_settings_val_exist(name, tolerate_blank=False, tolerate_spaces=True):
    val = get_settings_val(name)
    if val is None:
        raise Exception('Missing settings value: %s' % name)

    if (not tolerate_blank and val) == '' or (not tolerate_spaces and val.isspace()):
        raise Exception('Improperly configured settings value: %s' % name)


def is_func(obj):
    return hasattr(obj, '__call__')


def is_list(obj):
    if isinstance(obj, basestring):
        return False
    return isinstance(obj, collections.Sequence)


def convert_js_bool(o):
    if o:
        return 'true'
    return 'false'


def create_attr_from_obj(obj, ignore_function=True, ignore_lists=True, js_bool=True):
    attrs = dir(obj)
    d = {}
    for attr in attrs:
        val = getattr(obj, attr)
        if ignore_function and is_func(val):
            continue

        if ignore_lists and is_list(val):
            continue

        if val is True or val is False:
            val = convert_js_bool(val)

        d[attr] = val

    return d


#
#   Code Copied from https://github.com/un33k/django-ipware/blob/master/ipware/ip.py
#

import socket


def is_valid_ip(ip_address):
    """ Check Validity of an IP address """
    valid = True
    try:
        socket.inet_aton(ip_address.strip())
    except:
        valid = False
    return valid


def get_ip_address_from_request(request):
    """ Makes the best attempt to get the client's real IP or return the loopback """

    PRIVATE_IPS_PREFIX = ('10.', '172.', '192.', '127.' )
    ip_address = ''
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR', '')
    if x_forwarded_for and ',' not in x_forwarded_for:
        if not x_forwarded_for.startswith(PRIVATE_IPS_PREFIX) and is_valid_ip(x_forwarded_for):
            ip_address = x_forwarded_for.strip()
    else:
        ips = [ip.strip() for ip in x_forwarded_for.split(',')]
        for ip in ips:
            if ip.startswith(PRIVATE_IPS_PREFIX):
                continue
            elif not is_valid_ip(ip):
                continue
            else:
                ip_address = ip
                break
    if not ip_address:
        x_real_ip = request.META.get('HTTP_X_REAL_IP', '')
        if x_real_ip:
            if not x_real_ip.startswith(PRIVATE_IPS_PREFIX) and is_valid_ip(x_real_ip):
                ip_address = x_real_ip.strip()
    if not ip_address:
        remote_addr = request.META.get('REMOTE_ADDR', '')
        if remote_addr:
            if not remote_addr.startswith(PRIVATE_IPS_PREFIX) and is_valid_ip(remote_addr):
                ip_address = remote_addr.strip()
    if not ip_address:
        ip_address = '127.0.0.1'
    return ip_address


#
#   End copied code
#


# coding=utf-8
from django.conf.urls import url
from django.http import HttpResponse

from django_helpers import add_app_url
import urls

from data_table import DataTable

_URL_PREFIX = 'data-table-'

_TABLES = dict()


def has_registered(name):
    return name in _TABLES


def get_table(name):
    if name not in _TABLES:
        return None

    return _TABLES[name]()


def get_data(request, name, **kwargs):
    table = get_table(name)
    if table is None:
        return HttpResponse()

    results = table.get_data(request, kwargs)
    return results


def save(request, name, **kwargs):
    if request.method != 'POST':
        return HttpResponse()

    table = get_table(name)
    if table is None:
        return HttpResponse()

    results = table.save(request, kwargs)
    return results


def create_reg(name):
    return "(?P<%s>.*)/" % name


def _table_name(table):
    return '%s.%s' % (table.__module__, table.__name__)


def register(data_table):
    add_app_url('data-table', urls)

    if isinstance(data_table, DataTable):
        raise Exception("DataTable class is required not instance.")

    if not issubclass(data_table, DataTable):
        raise Exception("A Sub class of DataTable is required.")

    name = getattr(data_table, 'table_id', None)
    if name is None:
        raise Exception("Table ID is needed.")

    if not name.startswith(_URL_PREFIX):
        name = _URL_PREFIX + name

    if name in _TABLES:
        old_table = _TABLES[name]

        new_name = _table_name(data_table)
        old_name = _table_name(old_table)

        if old_name == new_name:
            return data_table

        msg = "You tried to register a data table with id " + name + " for " + new_name + ".\n"
        msg += "But " + name + " is already registered for " + old_name + ".\n"
        msg += "Try giving a new ID for " + new_name
        raise Exception(msg)

    setattr(data_table, 'table_id', name)
    _TABLES[name] = data_table
    data_reg = "%s/" % name
    save_reg = "%s/" % name
    extra_url_parameters = data_table.url_search_parameters
    if extra_url_parameters is not None:
        for parameter in extra_url_parameters:
            data_reg += create_reg(parameter)
            save_reg += create_reg(parameter)
    data_reg += "$"
    save_reg += "save/$"

    data_pattern = url(r"%s" % data_reg, get_data, name=name, kwargs={
        "name": name
    })

    save_pattern = url(r"%s" % save_reg, save, name=name + '_save', kwargs={
        "name": name
    })

    urls.urlpatterns.append(data_pattern)
    urls.urlpatterns.append(save_pattern)
    return data_table


__author__ = 'ajumell'

from json import dumps
from hashlib import md5

from django.http import HttpResponse
from django.conf.urls import url

from autocomplete import AutoComplete
import urls

_REGISTRY = dict()
url_prefix = 'auto-complete-'


def autocomplete_lookup(request, lookup, **kwargs):
    """

    @param request: django.http.HttpRequest
    @param lookup: name of the lookup
    @param kwargs: extra url parameters
    @return: django.http.HttpResponse
    """
    # If lookup does not exists then return blank array.
    # This is prevent any exceptions at runtime.
    if lookup not in _REGISTRY:
        return HttpResponse("[]")

    auto_complete = _REGISTRY[lookup]()

    results = auto_complete.get_results(request, kwargs)
    return HttpResponse(dumps(results))


def create_reg(name):
    return "(?P<%s>.*)/" % name


def register(autocomplete_class):
    if isinstance(autocomplete_class, AutoComplete):
        raise Exception("AutoComplete class is required not instance.")

    if not hasattr(autocomplete_class, 'name'):
        name = md5(unicode(autocomplete_class)).hexdigest()
    else:
        name = getattr(autocomplete_class, 'name')

    if not name.startswith(url_prefix):
        name = url_prefix + name

    setattr(autocomplete_class, 'name', name)

    if name in _REGISTRY:
        return

    _REGISTRY[name] = autocomplete_class
    reg = "%s/" % name
    extra_url_parameters = autocomplete_class.url_search_parameters
    if extra_url_parameters is not None:
        for parameter in extra_url_parameters:
            reg += create_reg(parameter)
    reg += "$"

    pattern = url(r"%s" % reg, autocomplete_lookup, name=name, kwargs={
        "lookup": name
    })
    urls.urlpatterns.append(pattern)
    return autocomplete_class


# coding=utf-8
from django.db import models
from django.contrib.auth.models import User
from django.utils.datetime_safe import datetime
from django.contrib.sessions.models import Session
from django_helpers import get_ip_address_from_request


class LoginHistoryManager(models.Manager):
    @property
    def active_sessions(self):
        return self.filter(session__expire_date__gte=datetime.now())

    @property
    def expired_sessions(self):
        return self.filter(session__expire_date__lt=datetime.now())

    def get_history(self, user):
        return self.active_sessions.filter(user=user)


class LoginHistory(models.Model):
    user = models.ForeignKey(User)
    timestamp = models.DateTimeField(auto_now_add=True)
    ip_address = models.IPAddressField(null=True)
    session = models.ForeignKey(Session)
    user_agent = models.CharField(max_length=255, blank=True)

    objects = LoginHistoryManager()

    def logout(self):
        self.session.delete()

# Connect signal
from django.dispatch import receiver
from django.contrib.auth.signals import user_logged_in


@receiver(user_logged_in)
def save_login_history(request, user):
    history = LoginHistory()
    history.user = user
    history.ip_address = get_ip_address_from_request(request)
    history.session = request.session
    history.user_agent = request.META['HTTP_USER_AGENT']
    history.save()



# coding=utf-8
from django.utils.translation import ugettext as _


class CSRFTokenMissingException(Exception):
    def __init__(self):
        Exception.__init__(self, _("Either request or csrf_token argument is needed."))


class FormIDMissingException(Exception):
    def __init__(self):
        Exception.__init__(self, _("Form ID is necessary"))



# coding=utf-8
from django.core.urlresolvers import reverse
from django.utils.safestring import mark_safe
from django.utils.text import capfirst
from django.db import models

from django_helpers.helpers.views import render_to_string
from models import Image
from django_helpers.apps.autocomplete import AutoComplete
from django_helpers.apps.autocomplete.fields import AutoCompleteField
from django_helpers.apps.autocomplete.widgets import AutoCompleteWidget


__author__ = 'ajumell'


class ImageManagerImageAutoComplete(AutoComplete):
    name = 'image-manager'
    query_set = Image.objects.all()
    search_fields = ['name']

    def format_value(self, instance):
        return instance.name

    def get_display_url(self, instance):
        return instance.thumbnail

    def get_results(self, request, params):
        term = self.get_search_term(request)
        query = self.search(self.query_set, term, params)
        results = []
        for result in query:
            val = self.format_value(result)
            results.append({
                "id": result.id,
                "label": val,
                "thumbnail": self.get_display_url(result)
            })
        return results


class ImageManagerImageSelectWidget(AutoCompleteWidget):
    has_js = False
    template = 'image-manager/image-selection-field.html'

    class Media:
        # TODO: Find a method to use admin jquery
        js = ('js/jquery-1.7.1.min.js', 'js/jquery-ui.js')

    def __init__(self, attrs=None, delay=0, min_length=0, lookup=None):
        AutoCompleteWidget.__init__(self, attrs, delay, min_length, lookup)
        # Fix for admin site
        self.choices = None

    def render(self, name, value, attrs=None):
        html = AutoCompleteWidget.render(self, name, value, attrs)
        auto_complete = self.auto_complete
        instance = auto_complete.get_instance(value)
        source = reverse(self.auto_complete.name)
        value = self.value or ""
        op = render_to_string(self.template, {
            "min_length": self.min_length,
            "delay": self.delay,
            "source": source,
            "id": self.html_id,
            "name": self.name,
            "label": self.formatted_value,
            "value": value,
            "image": auto_complete.get_display_url(instance),
            "html": mark_safe(html)
        })
        return op


# noinspection PyUnusedLocal
class ImageManagerImageSelectField(AutoCompleteField):
    widget = ImageManagerImageSelectWidget

    def __init__(self, lookup=None, required=True, widget=None, label=None, initial=None,
                 help_text=None, error_messages=None, show_hidden_initial=False,
                 validators=None, localize=False, queryset=None, to_field_name=None):
        if lookup is None:
            lookup = ImageManagerImageAutoComplete

        if queryset is not None:
            lookup.query_set = queryset

        AutoCompleteField.__init__(self, lookup, required, widget, label, initial, help_text,
                                   error_messages, show_hidden_initial, validators, localize)


class ImageManagerImageField(models.ForeignKey):
    def __init__(self, to_field=None, **kwargs):
        models.ForeignKey.__init__(self, Image, to_field, **kwargs)

    def formfield(self, **kwargs):
        db = kwargs.pop('using', None)
        defaults = {
            'required': not self.blank,
            'label': capfirst(self.verbose_name),
            'help_text': self.help_text,
            'queryset': self.rel.to._default_manager.using(db).complex_filter(self.rel.limit_choices_to)
        }

        if self.has_default():
            if callable(self.default):
                defaults['initial'] = self.default
                defaults['show_hidden_initial'] = True
            else:
                defaults['initial'] = self.get_default()

        val = ImageManagerImageSelectField(**defaults)
        return val


from types import StringTypes


def autodiscover(*names):
    """
    borrowed from django admin.
    """
    from django.conf import settings
    from django.utils.importlib import import_module
    from django.utils.module_loading import module_has_submodule

    for app in settings.INSTALLED_APPS:
        mod = import_module(app)
        for name in names:
            if type(name) not in StringTypes:
                raise
            try:
                import_module('%s.%s' % (app, name))
            except:
                if module_has_submodule(mod, name):
                    raise


# coding=utf-8
from django.forms import fields
from django.utils import formats
from django.utils.safestring import mark_safe

from . import Widget, js_bool, js_date
from django_helpers.helpers.views import render_to_string
from django_helpers.apps.static_manager import BootstrapURL, jQueryURL


class FormatNotAllowedException(Exception):
    def __init__(self, formats, part):
        msg = '"%s" is not allowed in this date format.\nAllowed formats are %s' % (part, formats)
        Exception.__init__(self, msg)


def _date_format_converter(django_date_format, formats):
    i = 0
    l = len(django_date_format)
    new_format = ''
    while i < l:
        first_letter = django_date_format[i]
        # If the character does not starts with %
        # Then its not a date format section.
        if first_letter != '%':
            new_format += first_letter
            i += 1
        else:
            # Python date formats are always two
            # characters with % as a start.
            section = django_date_format[i:i + 2]
            if section not in formats:
                if section == "%%":
                    new_format += "%"
                else:
                    raise FormatNotAllowedException(formats.keys(), section)
            else:
                new_format += formats[section]
            i += 2
    return new_format


def moment_date_format(django_date_format):
    """
    Django : http://docs.python.org/2/library/datetime.html#strftime-strptime-behavior
    Bootstrap : http://momentjs.com/docs/#/parsing/string-format/
    @param django_date_format:
    @return:
    """
    formats = {
        '%a': 'ddd',
        '%A': 'dddd',
        '%b': 'MMM',
        '%B': 'MMMM',
        '%d': 'DD',
        '%H': 'HH',
        '%I': 'hh',
        '%j': 'DDDD',
        '%m': 'MM',
        '%M': 'mm',
        '%p': 'A',
        '%S': 'ss',
        '%w': 'd',
        '%W': 'w',
        '%y': 'YY',
        '%Y': 'YYYY',
    }
    return _date_format_converter(django_date_format, formats)


class BootstrapDateFormatMixin():
    def js_date_format(self, date_format=None):
        """
        var val = {
                d: date.getUTCDate(),

                D: dates[language].daysShort[date.getUTCDay()],
                DD: dates[language].days[date.getUTCDay()],

                m: date.getUTCMonth() + 1,

                M: dates[language].monthsShort[date.getUTCMonth()],
                MM: dates[language].months[date.getUTCMonth()],

                yy: date.getUTCFullYear().toString().substring(2),
                yyyy: date.getUTCFullYear()
            };

        val.dd = (val.d < 10 ? '0' : '') + val.d;
        val.mm = (val.m < 10 ? '0' : '') + val.m;
        @return: Django date format converted to js format.
        """

        formats = {
            '%d': 'd',
            '%a': 'D',
            '%A': 'DD',
            '%m': 'm',
            '%b': 'M',
            '%B': 'MM',
            '%y': 'yy',
            '%Y': 'yyyy'
        }
        date_format = getattr(self, 'format', date_format)
        if date_format is None:
            raise Exception('Date format does not exists.')
        return _date_format_converter(date_format, formats)


# noinspection PyShadowingBuiltins
class DatePickerWidget(Widget, fields.DateInput, BootstrapDateFormatMixin):
    def __init__(self, attrs=None, format=None, week_start=0, calendar_weeks=False,
                 start_date=None, end_date=None, day_of_week_disabled=None, auto_close=False,
                 start_view=0, min_view_mode=0, today_btn=False, today_highlight=False,
                 keyboard_navigation=True, language='en', force_parse=True):
        fields.DateInput.__init__(self, attrs, format)

        self.js_files = [jQueryURL, BootstrapURL, 'js/moment.min.js', 'js/bootstrap-datepicker.js']

        self.week_start = week_start
        self.calendar_weeks = calendar_weeks
        self.start_date = start_date
        self.end_date = end_date
        self.day_of_week_disabled = day_of_week_disabled
        self.auto_close = auto_close
        self.start_view = start_view
        self.min_view_mode = min_view_mode
        self.today_btn = today_btn
        self.today_highlight = today_highlight
        self.keyboard_navigation = keyboard_navigation
        self.language = language
        self.force_parse = force_parse

    def render_js(self):
        return render_to_string('django-helpers/forms/bootstrap-date-picker.js', {
            "week_start": self.week_start,
            "calendar_weeks": js_bool(self.calendar_weeks),
            "id": self.html_id,
            "format": self.js_date_format(),
            "start_date": js_date(self.start_date),
            "end_date": js_date(self.end_date),
            "day_of_week_disabled": self.day_of_week_disabled,
            "auto_close": js_bool(self.auto_close),
            "start_view": self.start_view,
            "min_view_mode": self.min_view_mode,
            "today_btn": js_bool(self.today_btn),
            "today_highlight": js_bool(self.today_highlight),
            "keyboard_navigation": js_bool(self.keyboard_navigation),
            "language": self.language,
            "force_parse": js_bool(self.force_parse)
        })


# noinspection PyShadowingBuiltins
class ClockFaceWidget(Widget, fields.TimeInput):
    def __init__(self, attrs=None, format='H:mm', trigger='focus'):
        fields.TimeInput.__init__(self, attrs, format)

        self.js_files = [jQueryURL, BootstrapURL, 'js/clockface.js']

        self.trigger = trigger

    def js_date_format(self):
        formats = {
            '%H': 'HH',
            '%I': 'hh',
            '%M': 'mm',
            '%p': 'A'
        }
        return _date_format_converter(self.format, formats)

    def render_js(self):
        return render_to_string('django-helpers/forms/bootstrap-clock-face.js', {
            "id": self.html_id,
            "format": self.js_date_format(),
            "trigger": self.trigger
        })


# noinspection PyShadowingBuiltins
class DateTimePickerWidget(Widget, fields.DateTimeInput):
    def __init__(self, attrs=None, format=None, mask_input=True, pick_date=True,
                 pick_12_hour_format=False, pick_seconds=True, start_date=None, end_date=None):
        fields.DateTimeInput.__init__(self, attrs, format)
        self.js_files = [jQueryURL, BootstrapURL, 'js/bootstrap-datetimepicker.js']

        self.mask_input = mask_input
        self.pick_date = pick_date
        self.pick_12_hour_format = pick_12_hour_format
        self.pick_seconds = pick_seconds
        self.start_date = start_date
        self.end_date = end_date

    def js_date_format(self):
        """
          var dateFormatComponents = {
            dd: {property: 'UTCDate', getPattern: function() { return '(0?[1-9]|[1-2][0-9]|3[0-1])\\b';}},
            MM: {property: 'UTCMonth', getPattern: function() {return '(0?[1-9]|1[0-2])\\b';}},
            yy: {property: 'UTCYear', getPattern: function() {return '(\\d{2})\\b'}},
            yyyy: {property: 'UTCFullYear', getPattern: function() {return '(\\d{4})\\b';}},
            hh: {property: 'UTCHours', getPattern: function() {return '(0?[0-9]|1[0-9]|2[0-3])\\b';}},
            mm: {property: 'UTCMinutes', getPattern: function() {return '(0?[0-9]|[1-5][0-9])\\b';}},
            ss: {property: 'UTCSeconds', getPattern: function() {return '(0?[0-9]|[1-5][0-9])\\b';}},
            ms: {property: 'UTCMilliseconds', getPattern: function() {return '([0-9]{1,3})\\b';}},
            ms is not available in python
            HH: {property: 'Hours12', getPattern: function() {return '(0?[1-9]|1[0-2])\\b';}},
            PP: {property: 'Period12', getPattern: function() {return '(AM|PM|am|pm|Am|aM|Pm|pM)\\b';}}
          };



        """
        formats = {
            '%d': 'dd',
            '%m': 'MM',
            '%y': 'YY',
            '%Y': 'YYYY',
            '%H': 'hh',
            '%M': 'mm',
            '%S': 'ss',
            '%f': 'ms',
            '%I': 'HH',
            '%P': 'PP'
        }
        return _date_format_converter(self.format, formats)

    def render_js(self):
        return render_to_string('django-helpers/forms/bootstrap-date-time-picker.js', {
            "id": self.html_id,
            "format": self.js_date_format(),
            "mask_input": self.mask_input,

            "pick_date": js_bool(self.pick_date),
            "pick_12_hour_format": js_bool(self.pick_12_hour_format),
            "pick_seconds": js_bool(self.pick_seconds),

            "start_date": self.start_date,
            "end_date": self.end_date,

        })


# noinspection PyShadowingBuiltins
class ColorPickerWidget(Widget, fields.TextInput):
    def __init__(self, attrs=None, format='hex'):
        fields.TextInput.__init__(self, attrs)
        self.js_files = [jQueryURL, BootstrapURL, 'js/bootstrap-colorpicker.js']
        self.format = format

    def render_js(self):
        return render_to_string('django-helpers/forms/bootstrap-color-picker.js', {
            "id": self.html_id,
            "format": self.format,
        })


# noinspection PyShadowingBuiltins
class SelectWidget(Widget, fields.Select):
    def __init__(self, attrs=None, choices=(), button_class='btn', button_width='auto',
                 button_container=None, selected_class='active',
                 max_height=0, include_select_all_option=False, select_all_text=False, select_all_value=None,
                 enable_filtering=False, filter_placeholder='Search', drop_right=False):
        fields.Select.__init__(self, attrs, choices)
        self.js_files = [jQueryURL, BootstrapURL, 'js/bootstrap-multiselect.js']

        self.button_class = button_class
        self.button_width = button_width
        self.button_container = button_container
        self.selected_class = selected_class
        self.max_height = max_height
        self.include_select_all_option = include_select_all_option
        self.select_all_text = select_all_text
        self.select_all_value = select_all_value
        self.enable_filtering = enable_filtering
        self.filter_placeholder = filter_placeholder
        self.drop_right = drop_right

    def render_js(self):
        button_container = self.button_container
        if button_container is not None:
            button_container = mark_safe(self.button_container)

        return render_to_string('django-helpers/forms/bootstrap-multi-select.js', {
            "id": self.html_id,
            "button_class": self.button_class,
            "button_width": self.button_width,
            "button_container": button_container,
            "selected_class": self.selected_class,
            "max_height": self.max_height,
            "include_select_all_option": js_bool(self.include_select_all_option),
            "select_all_text": self.select_all_text,
            "select_all_value": self.select_all_value,
            "enable_filtering": js_bool(self.enable_filtering),
            "filter_placeholder": self.filter_placeholder,
            "drop_right": js_bool(self.drop_right)
        })


# noinspection PyShadowingBuiltins
class MultiSelectWidget(SelectWidget, fields.SelectMultiple):
    pass


# noinspection PyShadowingBuiltins
class TimePickerWidget(Widget, fields.TimeInput):
    def __init__(self, attrs=None, format=None, mask_input=True, pick_12_hour_format=False, pick_seconds=True):
        fields.TimeInput.__init__(self, attrs, format)
        self.js_files = [jQueryURL, BootstrapURL, 'js/bootstrap-datetimepicker.js']

        self.mask_input = mask_input
        self.pick_date = False
        self.pick_12_hour_format = pick_12_hour_format
        self.pick_seconds = pick_seconds
        # self.start_date = start_date
        # self.end_date = end_date

    def js_date_format(self):
        """
          var dateFormatComponents = {
            dd: {property: 'UTCDate', getPattern: function() { return '(0?[1-9]|[1-2][0-9]|3[0-1])\\b';}},
            MM: {property: 'UTCMonth', getPattern: function() {return '(0?[1-9]|1[0-2])\\b';}},
            yy: {property: 'UTCYear', getPattern: function() {return '(\\d{2})\\b'}},
            yyyy: {property: 'UTCFullYear', getPattern: function() {return '(\\d{4})\\b';}},

            hh: {property: 'UTCHours', getPattern: function() {return '(0?[0-9]|1[0-9]|2[0-3])\\b';}},
            mm: {property: 'UTCMinutes', getPattern: function() {return '(0?[0-9]|[1-5][0-9])\\b';}},
            ss: {property: 'UTCSeconds', getPattern: function() {return '(0?[0-9]|[1-5][0-9])\\b';}},
            ms: {property: 'UTCMilliseconds', getPattern: function() {return '([0-9]{1,3})\\b';}},
            HH: {property: 'Hours12', getPattern: function() {return '(0?[1-9]|1[0-2])\\b';}},
            PP: {property: 'Period12', getPattern: function() {return '(AM|PM|am|pm|Am|aM|Pm|pM)\\b';}}
          };



        """
        formats = {
            '%H': 'hh',
            '%M': 'mm',
            '%S': 'ss',
            '%f': 'ms',
            '%I': 'HH',
            '%P': 'PP'
        }
        return _date_format_converter(self.format, formats)

    def render_js(self):
        return render_to_string('django-helpers/forms/bootstrap-date-time-picker.js', {
            "id": self.html_id,
            "format": self.js_date_format(),
            "mask_input": self.mask_input,

            "pick_date": js_bool(False),
            "pick_12_hour_format": js_bool(self.pick_12_hour_format),
            "pick_seconds": js_bool(self.pick_seconds),

            # "start_date": self.start_date,
            # "end_date": self.end_date,

        })


# noinspection PyShadowingBuiltins
class DateRangeInputWidget(Widget, fields.TextInput):
    def __init__(self, attrs=None, format=None):
        fields.TextInput.__init__(self, attrs)
        if format is None:
            self.format = formats.get_format('DATE_INPUT_FORMATS')[0]
        else:
            self.format = format

        self.js_files = [jQueryURL, BootstrapURL, 'js/daterangepicker.js']

    def render_js(self):
        return render_to_string('django-helpers/forms/bootstrap-date-range.js', {
            "id": self.html_id,
            "format": moment_date_format(self.format),
        })


class SwitchWidget(Widget, fields.CheckboxInput):
    def __init__(self, attrs=None, check_test=None, on_label=None, off_label=None, on_icon=None, off_icon=None):
        fields.CheckboxInput.__init__(self, attrs, check_test)
        self.js_files = [jQueryURL, BootstrapURL, 'js/bootstrapSwitch.js']

        self.on_label = on_label if on_label is not None else ''
        self.off_label = off_label if off_label is not None else ''

        if on_icon is not None:
            self.on_label = "<i class='icon-%s'></i>" % on_icon

        if off_icon is not None:
            self.off_label = "<i class='icon-%s'></i>" % off_icon

    def render_js(self):
        return render_to_string('django-helpers/forms/bootstrap-switch.js', {
            "id": self.html_id,
            'yes': self.on_label,
            'no': self.off_label
        })


from django.utils.text import capfirst
from data_table import DataTable
from columns import DataTableColumn


class ModelDataTable(DataTable):
    model = None
    fields = None

    default_editable = False
    default_searchable = True
    default_sortable = True

    searchable = []
    sortable = []
    editable = []
    types = {}

    def __init__(self, request=None):
        if self.model is None:
            raise

        model = self.model
        model_fields = model._meta.fields

        if self.fields is None:
            fields = [model_field.name for model_field in model_fields]
        else:
            fields = self.fields

        columns = []
        types = self.types

        searchable = self.searchable
        sortable = self.sortable
        editable = self.editable
        need_related = False

        for model_field in model_fields:
            field = model_field.name
            if field in fields:
                Column = types.get(model_field, DataTableColumn)
                related = hasattr(model_field, 'related')

                need_related = need_related or related

                is_sortable = field in sortable or self.default_sortable
                is_editable = not related and (field in editable or self.default_editable)
                is_searchable = not related and (field in searchable or self.default_searchable)

                params = {
                    'searchable': is_searchable,
                    'sortable': is_sortable,
                    'editable': is_editable,
                    'title': capfirst(model_field.verbose_name),
                    'field': field,

                }
                columns.append(Column(**params))

        self.columns = columns
        self.query = model.objects.all()

        if need_related:
            self.query = self.query.select_related()

        DataTable.__init__(self, request)


# coding=utf-8
from renderer import FormRenderer
from preprocessors.bootstrap import append_preprocessor, prepend_preprocessor


class BootstrapFormRenderer(FormRenderer):
    template = 'form-renderer/bootstrap.html'

    def __init__(self, form, request=None, form_id=None, csrf_token=None, template=None):
        FormRenderer.__init__(self, form, request, form_id, csrf_token, template)
        if self.extra_context_dict is None:
            self.extra_context_dict = {}

        self.pre_processors.append(append_preprocessor)
        self.pre_processors.append(prepend_preprocessor)

        self.extra_context_dict['button_color'] = 'blue'
        self.extra_context_dict['layout'] = 'horizontal'

    @property
    def button_color(self):
        return self.extra_context_dict.get('button_color')

    @button_color.setter
    def button_color(self, value):
        self.extra_context_dict['button_color'] = value

    @property
    def layout(self):
        return self.extra_context_dict.get('layout')

    @layout.setter
    def layout(self, value):
        self.extra_context_dict['layout'] = value




# coding=utf-8

from data_table import DataTable
from views import register
from columns import DataTableColumn, DataTableTemplateColumn
from extras import BootstrapButtonColumn, BootstrapButtonSetColumn, DateDataTableColumn, EmailDataTableColumn, LinkDataTableColumn
from humanize import LineBreaksColumn, NaturalDayColumn, ComaSeparatedIntegerColumn, NaturalTimeColumn, OrdinalIntegerColumn
from models import ModelDataTable

__author__ = "ajumell"


from django_helpers.helpers.views import render_to_string
from django_helpers.apps.static_manager import jQueryURL, BootstrapURL

__author__ = 'ajumell'

from . import Menu


class BootstrapPillMenu(Menu):
    nav_type = 'pills'
    stacked = False

    has_js = True

    def __init__(self, context, items=None):
        Menu.__init__(self, context, items)
        self.js_files = [jQueryURL, BootstrapURL, 'js/jquery.django.ajax.js']

    def render_menu_start(self, level):
        if level is 0:
            classes = self.menu_classes[:]
            classes.insert(0, 'nav')
            classes.insert(1, 'nav-' + self.nav_type)

            if self.stacked:
                classes.insert(2, 'nav-stacked')

            return '<ul class="%s">' % ' '.join(classes)
        else:
            return '<ul class="dropdown-menu">'

    def get_template(self, level, has_sub_menu, decedent_current, is_current):
        if level == 0:
            if has_sub_menu:
                return 'menu/bootstrap-nav-with-drop-down.html'
            else:
                return 'menu/bootstrap-nav.html'

        if level > 0:
            if has_sub_menu:
                return 'menu/bootstrap-drop-down-item-with-sub.html'
            else:
                return 'menu/bootstrap-drop-down-item.html'

    def render_js(self):
        op = render_to_string('menu/bootstrap-dropdown.js', {
            'menu_id': self.menu_id,
            'ajax': self.ajax
        })
        return op


# coding=utf-8
"""

0 or 'none' - No transition effect
1 or 'fade' - Fade effect (Default)
2 or 'slideTop' - Slide in from top
3 or 'slideRight' - Slide in from right
4 or 'slideBottom' - Slide in from bottom
5 or 'slideLeft' - Slide in from left
6 or 'carouselRight' - Carousel from right to left
7 or 'carouselLeft' - Carousel from left to right


"""
from django_helpers import create_attr_from_obj, get_settings_val
from django_helpers.helpers.views import render_to_string
from django_helpers.apps.static_manager import add_css_file, jQueryUIURL, jQueryEasingURL

__author__ = 'ajumell'


class SuperSizedTransitions(object):
    none = 'none'
    fade = 'fade'
    slideTop = 'slideTop'
    slideRight = 'slideRight'
    slideLeft = 'slideLeft'
    slideBottom = 'slideBottom'
    carouselRight = 'carouselRight'
    carouselLeft = 'carouselLeft'


TransitionChoices = []

for item in dir(SuperSizedTransitions):
    if not item.startswith('__'):
        val = getattr(SuperSizedTransitions, item)
        if TransitionChoices.count(val) == 0:
            TransitionChoices.append((item, val))


class SuperSized(object):
    autoplay = True
    fit_always = False
    fit_landscape = False
    fit_portrait = True
    horizontal_center = False
    image_protect = True
    keyboard_nav = True
    min_height = 0
    min_width = 0
    new_window = True
    pause_hover = False
    performance = 1
    random = False
    slideshow = True
    slide_interval = 5000
    slide_links = False
    start_slide = 1
    stop_loop = False
    thumb_links = True
    thumbnail_navigation = True
    transition = SuperSizedTransitions.fade
    transition_speed = 750
    vertical_center = True
    progress_bar = True
    mouse_scrub = True

    show_thumbnails = False
    hide_controls = False

    def __init__(self):
        self.js_files = [
            jQueryUIURL,
            jQueryEasingURL,
            'super-sized/js/supersized.3.2.7.min.js',
            'super-sized/theme/supersized.shutter.min.js'
        ]

        add_css_file('super-sized/css/supersized.css')
        add_css_file('super-sized/theme/supersized.shutter.css')
        self._slides = []

    def add_slide(self, image, title, thumb, link):
        if not thumb:
            thumb = image

        self._slides.append({
            'image': image,
            'title': title,
            'thumb': thumb,
            'url': link
        })

    def render(self):
        attrs = create_attr_from_obj(self)
        attrs['slides'] = self._slides
        attrs['show_thumbnails'] = self.show_thumbnails
        attrs['hide_controls'] = self.hide_controls
        attrs['STATIC_URL'] = get_settings_val('STATIC_URL', '')
        return render_to_string('super-sized.html', attrs)

    def __unicode__(self):
        return self.render()

    def __str__(self):
        return self.render()


import datetime

from django.core.urlresolvers import reverse
from django.db import router
from django.db.models import ForeignKey, Model
from django.forms.util import from_current_timezone
from django.utils import formats, six, datetime_safe
from django.utils.encoding import force_text

from django_helpers.apps.forms.widgets.bootstrap import bootstrap_date_format
from django_helpers.helpers.views import render_to_string
from django_helpers.utils.html import generate_code

from js_imports import require_date_filed

#
#   This app will require forms app for some widgets.
#

input_list = """
    text
    textarea
    select
    date
    datetime
    dateui
    combodate
    html5types
    checklist
    wysihtml5
    typeahead
    select2
"""


class BaseEditable(object):
    html_tag = 'a'
    class_names = None
    js_files = None
    css_files = None
    title = ''

    has_rendered = False
    editable = None
    field_name = None
    js_template = None

    def __init__(self, required=None):
        self.required = required

    def _load_validations_from_model(self):
        field = self.get_model_field()
        self.required = not field.blank
        # TODO: Add more validations.

    def get_model_field(self):
        editable = self.editable
        return editable.get_model_field(self.field_name)

    def get_model_field_type(self):
        editable = self.editable
        return editable.get_model_field_type(self.field_name)

    def render(self):
        editable = self.editable
        if editable is None:
            raise

        self.has_rendered = True
        name = self.field_name
        field_id = editable.get_id(name)
        value = editable.get_value(name)
        text = self.to_string(value)
        attrs = {
            'id': field_id
        }
        if self.html_tag == 'a':
            attrs['href'] = '#'

        return generate_code(self.html_tag, attrs, text)

    def render_js(self):
        data = self.generate_render_data()
        return render_to_string(self.js_template, data)

    def __unicode__(self):
        return self.render()

    def __str__(self):
        return self.render()

    def __repr__(self):
        return self.render()

    def get_id(self):
        name = self.field_name
        return self.editable.get_id(name)

    def generate_render_data(self):
        editable = self.editable
        return {
            'pk': editable.pk,
            'title': self.title,
            'name': self.field_name,
            'field_id': self.get_id(),
            'save_url': reverse(editable.editable_id + "_save")
        }

    def to_python(self, value):
        """
        This function will handle the convertion of the
        string to its corresponding python object.
        @param value: The string value given from the http post
        @return: the converted value. In normal cases it will be same.
        """
        return value

    def to_string(self, value):
        """
        This function will convert the python object to
        the string representation
        @param value: python object
        @return: string representation of the python object
        """
        return value


class TextEditable(BaseEditable):
    placeholder = ''
    clear = True
    js_template = 'x-editable/text.js'

    def generate_render_data(self):
        data = BaseEditable.generate_render_data(self)
        data['placeholder'] = self.placeholder
        data['clear'] = self.clear
        return data


class TextAreaEditable(TextEditable):
    rows = 7
    js_template = 'x-editable/textarea.js'

    def generate_render_data(self):
        data = TextEditable.generate_render_data(self)
        data['rows'] = self.rows
        return data


class SelectEditable(BaseEditable):
    source_cache = True
    source_error = "Error occured when loading data."
    prepend = False
    source = None


class BaseTemporalField(TextEditable):
    date_format = None

    def to_python(self, value):
        # Model cannot handle the conversion since it does not
        # know the format.
        unicode_value = force_text(value, strings_only=True)
        if isinstance(unicode_value, six.text_type):
            value = unicode_value.strip()

        if isinstance(value, six.text_type):
            try:
                return self.strptime(value, self.date_format)
            except (ValueError, TypeError):
                raise
        raise Exception('Invalid format')

    # noinspection PyShadowingBuiltins
    def strptime(self, value, format):
        field_type = self.get_model_field_type()
        val = datetime.datetime.strptime(value, format)
        if field_type == 'DateField':
            val = val.date()
        elif field_type == 'TimeField':
            val = val.time()
        else:
            val = from_current_timezone(val)
        return val

    def to_string(self, value):
        if hasattr(value, 'strftime'):
            value = datetime_safe.new_datetime(value)
            return value.strftime(self.date_format)
        return value


class DatePickerEditable(BaseTemporalField):
    js_template = 'x-editable/date.js'
    js_files = require_date_filed()

    def __init__(self, date_format=None, view_format=None):
        TextEditable.__init__(self)

        if date_format is None:
            date_format = formats.get_format('DATE_INPUT_FORMATS')[0]

        if view_format is None:
            view_format = date_format

        self.date_format = date_format
        self.view_format = view_format

    def generate_render_data(self):
        data = TextEditable.generate_render_data(self)
        data['date_format'] = bootstrap_date_format(self.date_format)
        data['view_format'] = bootstrap_date_format(self.view_format)
        return data


class BaseChoiceField(BaseEditable):
    def __init__(self, required=None, choices=None, queryset=None):
        BaseEditable.__init__(self, required)
        self.choices = choices
        self.queryset = queryset

    def _get_choices(self):
        choices = self.choices
        queryset = self.queryset

        model_field = self.get_model_field()

        if choices is None and queryset is None:
            # Find if model has relation or choices
            is_foreign = isinstance(model_field, ForeignKey)
            if not is_foreign:
                # Borrowed from the django code base.
                choices = model_field.choices
                self.choices = choices
                return choices
            else:
                # Borrowed from the django code base.
                # django/db/models/fields/related.py Line 1088, v:1.5.1
                # TODO Find a method to implement using multiple db.
                queryset = model_field.rel.to._default_manager.using(None).complex_filter(model_field.rel.limit_choices_to),
                if type(queryset) is tuple and len(queryset) == 1:
                    queryset = queryset[0]
                self.queryset = queryset
                return queryset
        elif choices is None and queryset is not None:
            return queryset
        elif choices is not None and queryset is None:
            return choices
        else:
            raise

    def list(self):
        result = []
        choices = self._get_choices()
        for choice in choices:
            if type(choice) == tuple:
                pk = choice[0]
                value = choice[1]
            elif isinstance(choice, Model):
                pk = choice.pk
                value = str(choice)
            else:
                raise Exception('Instance is an instance of %s not Model. ' % type(choice) + str(choice))

            result.append({
                'value': pk,
                'text': value
            })
        return result

    def generate_render_data(self):
        editable = self.editable
        data = BaseEditable.generate_render_data(self)
        data['source'] = reverse(editable.editable_id + "_list", kwargs={
            'field_name': self.field_name
        })
        return data


class SimpleSelectField(BaseChoiceField):
    js_template = 'x-editable/select.js'


class ModelSelectField(SimpleSelectField):
    def to_python(self, value):
        field = self.get_model_field()
        editable = self.editable
        model_instance = editable.get_instance()
        using = router.db_for_read(model_instance.__class__, instance=model_instance)
        qs = field.rel.to._default_manager.using(using).filter(
            **{field.rel.field_name: value}
        )
        return qs.complex_filter(field.rel.limit_choices_to)[0]

    def to_string(self, value):
        return unicode(value)



# coding=utf-8
from json import dumps
from math import ceil
from types import StringTypes, TypeType

from django.utils.safestring import mark_safe
from django.core.urlresolvers import reverse
from django.db.models import Q, Sum, Avg, Min, Max, Count
from django.http import HttpResponse
from django.template import loader, Context

from django_helpers.apps.static_manager import jQueryURL, AjaxCsrfURL
from django_helpers.utils.models import last_query


__author__ = 'ajumell'

_annotates = {
    'sum': Sum,
    'avg': Avg,
    'min': Min,
    'max': Max,
    'count': Count,
}


def validate_annotate(name):
    return name in _annotates


# noinspection PyMethodParameters
class DataTableMeta(type):
    def __new__(cls, name, bases, attrs):
        from columns import DataTableColumn

        new_class = type.__new__(cls, name, bases, attrs)

        # This can be done now. Now think about a better syntax.
        fields = []
        for attr_name, attr in attrs.iteritems():
            if isinstance(attr, DataTableColumn):
                fields.append(attr)

        # no_columns = '___no_columns___'
        # old_columns = getattr(new_class, 'columns', no_columns)

        # if old_columns not in (old_columns, None):
        #     warn(DeprecationWarning('New Columns methods should be used now.'))
        # elif len(fields) > 0:
        #     new_class.columns = fields

        return new_class


class DataTable(object):
    __metaclass__ = DataTableMeta

    send_sql = True

    aggregate_fields = []
    has_js = True
    query = None
    url_search_parameters = None

    width = None
    class_name = ""
    name = None
    columns = []
    table_id = None
    display_length = 10
    display_lengths = [2, 3, 5, 10, 25, 50, 100]
    loading_message = 'Please wait while data is loading'

    ui_themes = False
    bootstrap_theme = True

    scroller = False
    scroller_height = 200

    info_empty = None
    info_loading = None
    info_processing = None

    row_id = None

    dom = None
    template = 'data-tables/table-html.html'
    js_template = 'data-tables/table-js.js'

    render_initial = False

    form_filter = None

    def __init__(self, request=None):

        if len(self.columns) is 0:
            raise Exception('There are no columns.')

        self.request = request

        js_files = [
            jQueryURL,
            AjaxCsrfURL,
            'data-tables/js/jquery-data-tables-min.1.9.4.js'
        ]

        if self.bootstrap_theme:
            js_files.append('data-tables/js/DT_bootstrap.js')

        self.columns = self.columns[:]

        if self.scroller:
            js_files.append('data-tables/js/dataTables.scroller.min.js')

        if self.scroller:
            js_files.append('data-tables/js/dataTables.scroller.min.js')

        if self.is_editable():
            js_files.append('js/jquery.jeditable.js')
            js_files.append('data-tables/js/dataTable.editable.js')

        if self.send_sql:
            js_files.append('data-tables/js/dataTable.sql.js')
            js_files.append('js/prettify.min.js')

        self.js_files = js_files

        self._data = None

        filter_form = self.form_filter
        if filter_form is not None:
            if type(filter_form) is TypeType:
                filter_form = filter_form()
                self.form_filter = filter_form

            if filter_form.form_id is None:
                raise

    def __unicode__(self):
        return self.render()

    def is_editable(self):
        if not hasattr(self, '_is_editable'):
            is_editable = False
            editable_columns = []
            i = 0
            for column in self.columns:
                i += 1
                if column.editable is True:
                    is_editable = True
                    editable_columns.append(i)
            self._is_editable = is_editable
            self._editable_columns = editable_columns
        return self._is_editable

    def generate_render_data(self, kwargs=None, args=None):
        if self._data is None:

            render_initial = self.render_initial
            if render_initial and args is not None:
                raise Exception('Arguments cannot be used when using initial data')

            is_editable = self.is_editable()
            table_id = self.table_id
            try:
                ajax_source = reverse(self.table_id, kwargs=kwargs, args=args)
                save_source = reverse(self.table_id + '_save', kwargs=kwargs, args=args)
            except Exception:
                from views import has_registered

                if not has_registered(table_id):
                    t = type(self)
                    m = t.__module__
                    n = t.__name__
                    raise Exception('This table "%s.%s" is not registered.' % (m, n))
                raise Exception('Django helpers is not added in root urls.')

            self._data = {
                'display_length': self.display_length,
                'display_lengths': self.display_lengths,
                'dom': self.dom,
                'ajax_source': ajax_source,
                'save_source': save_source,
                'columns': self.columns,
                "table_id": table_id,
                'width': self.width,
                'loading_message': self.loading_message,

                "ui_themes": self.ui_themes,
                "bootstrap_theme": self.bootstrap_theme,

                'is_editable': is_editable,

                'scroller': self.scroller,
                'scroller_height': self.scroller_height,

                'info_loading': self.info_loading,
                'info_processing': self.info_processing,
                'info_empty': self.info_empty,

                'initial_data': render_initial
            }

            filter_form = self.form_filter
            if filter_form is not None:
                self._data['has_form'] = True
                self._data['form_id'] = filter_form.form_id

            if is_editable:
                self._data['editable_columns'] = self._editable_columns

            if render_initial:
                self.initial_data(kwargs)

        return self._data

    def render_template(self, template, kwargs=None, args=None):
        template = loader.get_template(template)
        context = Context(self.generate_render_data(kwargs, args))
        return template.render(context)

    def render_js(self):
        return self.render_template(self.js_template)

    def render(self):
        return self.render_template(self.template)

    def safe_string(self, kwargs=None, args=None):
        self.generate_render_data(kwargs, args)
        return mark_safe(self.render())

    # noinspection PyUnusedLocal
    def get_query(self, request, kwargs=None):
        if hasattr(self, 'query'):
            return self.query

    def process_extra_params(self, query, kwargs):
        extra_params = self.url_search_parameters
        if extra_params is not None:
            filter_dict = {}
            for param_name in extra_params:
                param_value = kwargs.get(param_name, '')
                if param_value != '':
                    filter_dict[param_name] = param_value
            query = query.filter(**filter_dict)
        return query

    def annotate(self, query):
        columns = self.columns
        for column in columns:
            field = column.field
            annotate = column.annotate

            if annotate is not None:
                i = field.rfind('__')
                field = field[:i]
                fn = _annotates[annotate]
                query = query.annotate(fn(field))
        return query

    def _get_query(self, request, kwargs=None):
        query = self.get_query(request, kwargs)
        query = self.process_extra_params(query, kwargs)
        query = self.annotate(query)
        return query

    def sort_first_column(self, query):
        column = self.columns[0]
        if column.sortable is False:
            return query
        return self.sort(query, sortings=[column.field])

    def sort(self, query, DICT=None, sortings=None):
        if sortings is None and DICT is not None:
            gt = DICT.get
            sortings = []
            columns = self.columns
            total_sort_columns = gt('iSortingCols')
            total_sort_columns = int(total_sort_columns)

            if total_sort_columns is 0:
                return None

            while total_sort_columns:
                total_sort_columns -= 1
                column_number = int(gt('iSortCol_%d' % total_sort_columns))
                sort_order = gt('sSortDir_%d' % total_sort_columns)
                column = columns[column_number]
                if column.sortable:
                    field = column.field
                    if sort_order == 'desc':
                        field = '-' + field
                    sortings.append(field)
            sortings.reverse()

        if sortings is not None:
            query = query.order_by(*sortings)

        return query

    def page_data(self, query, DICT=None, start=None, max_items=None):
        if start is None and max_items is None:
            gt = DICT.get
            start = int(gt('iDisplayStart'))
            max_items = int(gt('iDisplayLength'))

        if max_items is None:
            max_items = self.display_length

        return query[start:start + max_items]

    def search(self, query, DICT=None, search_term=None):
        columns = self.columns

        if DICT is not None and search_term is None:
            gt = DICT.get
            search_term = gt('sSearch')

        search = None
        for column in columns:
            if column.searchable and search_term != "":
                field = column.field + "__" + column.search_lookup
                kwargs = {field: search_term}
                if search is None:
                    search = Q(**kwargs)
                else:
                    search = search | Q(**kwargs)

        if search is not None:
            query = query.filter(search)

        return query

    def only(self, query):
        columns = self.columns
        row_id = self.row_id
        fields = []
        need_related = False
        aggregate_fields = self.aggregate_fields
        related_fields = []
        for column in columns:
            column_related = column.need_related()
            need_related = need_related or column_related

            get_fields = getattr(column, 'get_fields', None)
            if get_fields is None:
                field = column.field
                annotate = column.annotate

                if annotate is None and field not in aggregate_fields:
                    fields.append(field)
                    if column_related:
                        related_fields.append(field)
            else:
                column_fields = get_fields()
                typ = type(column_fields)
                if typ in StringTypes:
                    fields.append(column_fields)
                elif typ in (tuple, list):
                    for column_field in column_fields:
                        fields.append(column_field)
                        if column_related:
                            related_fields.append(column_field)

        if row_id is not None:
            fields.insert(0, row_id)

        fields = tuple(fields)
        related_fields = tuple(related_fields)

        if need_related:
            query = query.select_related(*related_fields)

        query = query.only(*fields)
        return query

    #
    #   Initial Data For Table rendering
    #
    def initial_data(self, kwargs):
        request = self.request

        if self.request is None:
            raise Exception('Request is required for rendering initial data')

        query = self._get_query(request, kwargs)

        search_term = self.initial_search_term(request)
        prev_total = None
        if search_term:
            prev_total = query.count()
            query = self.search(query, search_term=search_term)

        query = self.sort_first_column(query)
        total_length = query.count()
        paginator = self.initial_pagination(request, total_length)
        query = self.only(query)
        query = self.page_data(query, start=paginator['from'] - 1)

        data = self._data

        data['rows'] = self.generate_initial_data_dict(query, request)
        data['total'] = total_length
        data['qs'] = paginator['qs']
        data['request'] = request
        data['paginator'] = paginator
        data['search_term'] = search_term
        data['prev_total'] = prev_total

        if self.send_sql:
            data['sql'] = last_query()

    def generate_initial_data_dict(self, query, request):
        rows = []
        row_id = self.row_id
        columns = self.columns
        for record in query:
            row = {}
            if row_id is not None:
                row['row_id'] = getattr(record, row_id)
            cells = []
            row['cells'] = cells
            for column in columns:
                cells.append(mark_safe(column.get_html(request, record)))
            rows.append(row)

        return rows

    # noinspection PyShadowingBuiltins
    def initial_pagination(self, request, count, page=None):
        qs = self.table_id + '-page'

        if page is None:
            try:
                page = request.GET.get(qs, 1)
                page = int(page)
            except:
                page = 1

        per_page = self.display_length
        hits = max(1, count)
        num_pages = int(ceil(hits / float(per_page)))
        prev_pages = []
        next_pages = []

        next_count = 4
        prev = page - 1

        while len(prev_pages) < 2 and prev > 0:
            if prev == 1:
                prev_pages.append((prev, None))
                break
            prev_pages.append((prev, prev))
            next_count -= 1
            prev -= 1
        prev_pages.reverse()

        next = page + 1
        while len(next_pages) < next_count and next <= num_pages:
            next_pages.append((next, next))
            next += 1

        paginator = {
            'from': per_page * (page - 1) + 1,
            'to': count if page == num_pages else page * per_page,
            'has_next': page < num_pages,
            'has_prev': page > 1,

            'current': page,

            'next_page': page + 1,
            'next_pages': next_pages,

            'prev_pages': prev_pages,
            'prev_page': None if page - 1 == 1 else page - 1,

            'qs': qs
        }

        return paginator

    def initial_search_term(self, request):
        search_key = self.table_id + '-search'
        search_term = request.GET.get(search_key, None)
        return search_term

    def initial_search(self, request, query):
        search_key = self.table_id + '-search'
        search_term = request.GET.get(search_key, None)
        if search_term is not None:
            query = self.search(query, search_term=search_term)
        return search_term, query

    def do_form_filter(self, query, DICT):
        form_filter = self.form_filter
        if form_filter is not None:
            query = form_filter.filter(query, DICT)
        return query

    #
    #   AJAX Table Rendering
    #
    def get_data(self, request, kwargs):
        DICT = request.GET
        query = self._get_query(request, kwargs)
        total_length = query.count()
        query = self.search(query, DICT)
        query = self.do_form_filter(query, DICT)
        query = self.sort(query, DICT)
        query = self.only(query)
        current_length = query.count()
        query = self.page_data(query, DICT)
        result = self.generate_ajax_json(query, request, DICT, total_length, current_length)
        return HttpResponse(dumps(result))

    def generate_ajax_json(self, query, request, DICT, total_length, current_length):
        datas = []
        row_id = self.row_id
        columns = self.columns
        gt = DICT.get
        echo = gt('sEcho')

        for record in query:
            data = {}

            if row_id is not None:
                data['DT_RowId'] = getattr(record, row_id)

            i = 0
            for column in columns:
                data[str(i)] = unicode(column.get_html(request, record))
                i += 1
            datas.append(data)

        data = {
            "aaData": datas,
            "sEcho": echo,
            "iTotalRecords": total_length,
            "iTotalDisplayRecords": current_length
        }

        if self.send_sql:
            data['sql'] = last_query()

        return data

    #
    #   Editable Data Table
    #
    def save_object(self, query, value, row_id, field):
        query = query.filter(**{
            self.row_id: row_id
        })

        query.update(**{
            field: value
        })

        return HttpResponse(value + ' - Saved')

    def save(self, request, kwargs=None):
        query = self.get_query(request, kwargs)
        query = self.process_extra_params(query, kwargs)

        gt = request.POST.get
        value = gt('value')
        row_id = gt('row_id')
        column = int(gt('column'))
        column = self.columns[column]
        field = column.field

        if field is None:
            raise Exception('Data cannot be saved in a column with out field')

        if row_id is None:
            raise Exception('Data cannot be saved with out row ID')

        if value is not None and row_id is not None:
            return self.save_object(query, value, row_id, field)



from urlparse import urlparse, urlunparse

from django.http import QueryDict
from django import template
from django.template import TemplateSyntaxError
from django_helpers.helpers.templatetags import parse_args, TemplateNode


register = template.Library()

# Copied from https://github.com/fatbox/fatbox-django-utils/blob/master/fatbox_utils/templatetags/urltools.py


@register.simple_tag(takes_context=True)
def abs_uri(context, relative_url):
    if 'request' not in context:
        return relative_url
    return context['request'].build_absolute_uri(relative_url)


@register.simple_tag(takes_context=True)
def modify_qs(context, **kwargs):
    if 'request' not in context:
        return 'You need to enable the request context processor!'

    # thanks to: http://stackoverflow.com/a/12423248
    # updated to allow None to delete an element
    url = context['request'].get_full_path()
    (scheme, netloc, path, params, query, fragment) = urlparse(url)
    query_dict = QueryDict(query).copy()
    for key in kwargs:
        query_dict_key = key
        if key in context:
            query_dict_key = context[key]

        if kwargs[key] is None:
            if query_dict_key in query_dict:
                del query_dict[query_dict_key]
        else:
            query_dict[query_dict_key] = kwargs[key]
    query = query_dict.urlencode()
    return urlunparse((scheme, netloc, path, params, query, fragment))


# class URLNode(TemplateNode):
#     def render(self, context):
#         if 'request' not in context:
#             return 'You need to enable the request context processor!'
#         url = context['request'].get_full_path()
#         print self.kwarg, self.args
#         return url
#
#
# @register.tag
# def modify_qs(parser, token):
#     bits = token.split_contents()
#     print bits
#     if len(bits) < 2:
#         raise TemplateSyntaxError("'%s' needs at least two arguments." % bits[0])
#     args, kwargs = parse_args(bits, parser)
#     return URLNode(args, kwargs)


# coding=utf-8
from django.conf.urls import patterns, url
import views


urlpatterns = patterns('',
    url(r'^$', views.index, name='image-manager-home'),
    url(r'^delete/(?P<image_id>\d+)-(?P<image_slug>.+)/$', views.delete_image, name='image-manager-delete-image'),
)



from django.template import TemplateSyntaxError, Library
from django.template.loader import render_to_string
from django.template.loader_tags import BlockNode
from django_helpers.apps.static_manager import jQueryURL
from django_helpers.helpers.templatetags import parse_args, resolve_args
from django_helpers.utils.html import generate_code

register = Library()


class AJAXBlockRegister(list):
    has_js = True
    is_bare_js = True

    def __init__(self):
        list.__init__(self)
        self.js_files = [jQueryURL, 'js/jquery.django.ajax.js']

    def render_js(self):
        return render_to_string('ajax-block/blocks.js', {
            'blocks': self
        })

    def __unicode__(self):
        return 'AJAX Block Register'


def get_block_id(name):
    return 'ajax-block-' + name


class AJAXBlockNode(BlockNode):
    def __init__(self, name, nodelist, parent=None, args=None, kwargs=None):
        BlockNode.__init__(self, name, nodelist, parent)
        self.kwargs = kwargs
        self.args = args

    def get_id(self):
        return get_block_id(self.name)

    def resolver_tag(self, context):
        args = self.args
        kwargs = self.kwargs

        if len(args) > 0:
            tag = args[0]
            if 'tag' in kwargs:
                raise TemplateSyntaxError('Tag occures twice.')
        elif 'tag' in kwargs:
            tag = kwargs.pop('tag')
        else:
            raise TemplateSyntaxError('Tag is not specified.')

        if 'id' in kwargs:
            raise TemplateSyntaxError('ID should not be passed as an argument. It will be automatically generated from block name.')

        self.tag = tag.resolve(context, True)

    def register_block_js(self, context):
        key = 'ajax_block_register'
        registry = context.get(key)
        if registry is None:
            registry = AJAXBlockRegister()
            context = context.dicts[-2]
            context[key] = registry
        elif not isinstance(registry, AJAXBlockRegister):
            raise
        registry.append((self.name, self.get_id()))

    def start_tag(self, context):
        self.resolver_tag(context)
        kwargs = resolve_args(None, self.kwargs, context)
        kwargs['id'] = self.get_id()
        return generate_code(self.tag, kwargs, start_only=True)

    def render(self, context):
        is_ajax = context.get('is_ajax_render', False)
        op = BlockNode.render(self, context)
        if not is_ajax:
            self.register_block_js(context)
            op = self.start_tag(context) + op
            op += '</%s>' % self.tag
        return op


@register.tag('ajax_block')
def _ajax_block(parser, token):
    """
    Code Borrowed from Django Code Base.
    Define a block that can be overridden by child templates.
    """

    bits = token.contents.split()
    block_name = bits[1]
    args, kwargs = parse_args(bits[1:], parser)

    if len(args) > 1:
        raise TemplateSyntaxError('%s does not take more than two arguments.')

    try:
        if block_name in parser.__loaded_blocks:
            raise TemplateSyntaxError("'%s' tag with name '%s' appears more than once" % (bits[0], block_name))
        parser.__loaded_blocks.append(block_name)
    except AttributeError:  # parser.__loaded_blocks isn't a list yet
        parser.__loaded_blocks = [block_name]
    nodelist = parser.parse(('endajax_block',))

    # This check is kept for backwards-compatibility. See #3100.
    endblock = parser.next_token()
    acceptable_endblocks = ('endajax_block', 'endajax_block %s' % block_name)
    if endblock.contents not in acceptable_endblocks:
        parser.invalid_block_tag(endblock, 'endajax_block', acceptable_endblocks)

    return AJAXBlockNode(block_name, nodelist, None, args, kwargs)


# coding=utf-8
import types

from django.utils.html import conditional_escape
from django.utils.safestring import mark_safe
from django_helpers.utils.html import generate_code


__author__ = 'ajumell'
__all__ = ('prepend_preprocessor', 'append_preprocessor')


def make_addon(text):
    return u'<span class="add-on">%s</span>' % conditional_escape(text)


def get_btn_class(color):
    if color != '':
        color = ' btn-' + color
    return 'btn' + color


def get_contents(text, icon):
    if icon != '':
        icon = generate_code('i', {
            'class': 'icon icon-' + icon
        }, '')
    return mark_safe(icon + conditional_escape(text))


def make_button(text, color='', button_type='button', icon=''):
    return generate_code('button', {
        'class': get_btn_class(color),
        'type': button_type
    }, get_contents(text, icon))


def make_link_button(text, url='#', color='', icon=''):
    return generate_code('a', {
        'class': get_btn_class(color),
        'href': url
    }, get_contents(text, icon))


def make_text(prop):
    name = 'addon'
    args = prop

    if isinstance(prop, dict):
        assert len(prop.keys()) == 1, 'There cannot be more than one property'
        name, args = prop.items()[0]

    elif type(prop) in (tuple, list):
        name = prop[0]
        args = prop[1:]
        assert len(args) > 0, 'Arguments must be greater than one'

    name = 'make_%s' % name
    if name not in globals():
        return

    func = globals()[name]

    if isinstance(func, types.FunctionType):
        if type(args) in (tuple, list):
            return func(*args)
        elif type(args) is dict:
            return func(**args)
        else:
            return func(args)


def _base_preprocessor(renderer, property_name='addend'):
    form = renderer.form
    datas = getattr(renderer, '%ss' % property_name, None)
    if type(datas) is not dict:
        return
    for name, data in datas.items():
        field = form.fields[name]
        setattr(field, 'has_%s' % property_name, True)
        if type(data) in (list, tuple):
            text = ''
            for prop in data:
                text += make_text(prop)
        else:
            text = make_text(data)
        setattr(field, '%s_text' % property_name, mark_safe(text))


def prepend_preprocessor(renderer):
    return _base_preprocessor(renderer, 'prepend')


def append_preprocessor(renderer):
    return _base_preprocessor(renderer, 'append')


# coding=utf-8
from storages.backends.s3boto import S3BotoStorage

__author__ = 'ajumell'

class S3Storage(S3BotoStorage):
    def __init__(self, *args, **kwargs):
        S3BotoStorage.__init__(self, *args, **kwargs)
        self.secure_urls = False


class SecureS3Storage(S3BotoStorage):
    pass



# coding=utf-8
from django.conf.urls import patterns, url
from preprocessors.validation import validate_form

urlpatterns = patterns(
    '',
    url(r'^validate-form/$', validate_form, name='validate-form'),
)



# coding=utf-8
from django.contrib import messages
from django_helpers.apps.image_manager.forms import AddImageFormRenderer, AddImageForm, Image
from django_helpers.helpers.views import render_to_response, redirect

__author__ = 'ajumell'

# require_jquery()
# require_jquery_ui()
# add_css_file('css/image-manager/main.css')
#
# add_js_file('fancybox/jquery.fancybox.pack.js')
# add_css_file('fancybox/jquery.fancybox.css')


def generate_token(param):
    return param


def index(request):
    if request.method == 'POST':
        form = AddImageForm(request.POST, request.FILES)
        print form.is_multipart()
        if form.is_valid():
            print 'Called...'
            form.save()
            messages.success(request, 'Image added successfully.')
            form = AddImageForm()
        else:
            pass
    else:
        form = AddImageForm()
    return render_to_response('image-manager/display-images.html', request, {
        'form': AddImageFormRenderer(form, request),
        'images': Image.objects.all()
    })


def delete_image(request, image_id, image_slug):
    token = generate_token("%s-%s" % (image_id, image_slug))
    image = Image.objects.get(id=image_id, name=image_slug)
    if request.method == 'POST':
        request_token = request.POST['token']
        if token == request_token:
            option = request.POST['submit']
            if option == 'yes':
                image.delete()
                messages.info(request, 'Image deleted.')
            else:
                messages.info(request, 'Image deletion cancelled.')
            return redirect(index)
        else:
            pass
    return render_to_response('image-manager/delete-confirm.html', request, {
        'token': token,
        'image': image
    })



__author__ = 'ajumell'

from editable import XEditable
from fields import TextEditable, TextAreaEditable, SelectEditable, DatePickerEditable, SimpleSelectField
from js_imports import require, require_date_filed




# coding=utf-8
__author__ = 'ajumell'

from django_helpers import get_settings_val
from django_helpers.helpers.templatetags import TemplateNode, parse_args
from django.contrib.staticfiles import finders
from django import template
from django.conf import settings
register = template.Library()

JS_FILES = []
CSS_FILES = []

STATIC_URL = get_settings_val('STATIC_URL')
INCLUDE_CSS = get_settings_val('STATIC_MANAGER_INCLUDE_CSS', True)
INCLUDE_JS = get_settings_val('STATIC_MANAGER_INCLUDE_JS', True)

__all__ = ('add_js_file', 'add_css_file', 'require_jquery', 'require_jquery_ui', 'require_bootstrap')


class StaticFileNotFoundException(Exception):
    def __init__(self, name):
        Exception.__init__(self, 'Static File "%s" not found.' % name)


def add_file(array, name):
    if array.count(name) == 0:
        if finders.find(name) is None:
            if getattr(settings, 'RAISE_STATIC_EXCEPTIONS', True):
                raise StaticFileNotFoundException(name)
            else:
                print StaticFileNotFoundException(name)
        else:
            array.append(name)


def add_js_file(name, force=False):
    if INCLUDE_JS or force:
        add_file(JS_FILES, name)


def add_css_file(name, force=False):
    if INCLUDE_CSS or force:
        add_file(CSS_FILES, name)

# TODO: Find a method to make this to one file
# TODO: Check absolute urls in included files

def get_code(template, array):
    code = []
    for f in array:
        if f.startswith("http://") or f.startswith("https://"):
            s = ""
        else:
            s = STATIC_URL
        code.append(template % (s, f))
    return '\n'.join(code)


def get_js_code():
    return get_code('<script type="text/javascript" src="%s%s"></script>', JS_FILES)


def get_css_code():
    return get_code('<link rel="stylesheet" href="%s%s" />', CSS_FILES)


class StaticFilesNode(TemplateNode):
    def render(self, context):
        mode = 'all'
        if len(self.args) > 0:
            mode = self.args[0].resolve(context, True)
            if mode not in ['css', 'js']:
                mode = 'all'

        if mode == 'all':
            return get_css_code() + '\n' + get_js_code()

        if mode == 'css':
            return get_css_code()

        if mode == 'js':
            return get_js_code()


@register.tag
def static_files(parser, token):
    bits = token.contents.split()
    args, kwargs = parse_args(bits, parser)
    return StaticFilesNode(args, kwargs)


def require_jquery():
    add_js_file('django-helpers/js/jquery-1.7.1.min.js')


def add_jquery_easing():
    add_js_file('django-helpers/js/jquery.easing.min.js')


def require_jquery_ui():
    add_js_file('django-helpers/js/jquery-ui.js')


def require_bootstrap():
    require_jquery()
    add_js_file('django-helpers/bootstrap/js/bootstrap.min.js')

    add_css_file('django-helpers/bootstrap/css/bootstrap.min.css')
    add_css_file('django-helpers/bootstrap/css/bootstrap-responsive.min.css')



from django import template
from django.utils.html import escapejs
from django.utils.safestring import mark_safe

register = template.Library()


def js_bool(value):
    if value is True:
        return 'true'
    if value is False:
        return 'false'
    return 'undefined'


def js_string(value):
    return mark_safe('"' + escapejs(value) + '"')


def jq_id(value):
    return mark_safe('"#' + escapejs(value) + '"')


def js_array(value):
    value = list(value)
    return mark_safe(str(value))


register.filter('js_string', js_string)
register.filter('jq_id', jq_id)
register.filter('js_array', js_array)
register.filter('js_bool', js_bool)


# coding=utf-8
__author__ = 'ajumell'

from django.forms import Form, ModelForm, ValidationError

from fields import CharField, IntegerField, DateField, TimeField, DateTimeField, TimeField, EmailField, FileField, ImageField, URLField, BooleanField, ChoiceField, MultipleChoiceField, FloatField, \
    DecimalField, IPAddressField, GenericIPAddressField, FilePathField, SlugField, TypedChoiceField, TypedMultipleChoiceField, SplitDateTimeField, ComboField, MultiValueField, NullBooleanField, \
    RegexField


__all__ = (
    'CharField',
    'IntegerField',
    'DateField',
    'TimeField',
    'DateTimeField',
    'TimeField',
    'EmailField',
    'FileField',
    'ImageField',
    'URLField',
    'BooleanField',
    'ChoiceField',
    'MultipleChoiceField',
    'FloatField',
    'DecimalField',
    'IPAddressField',
    'GenericIPAddressField',
    'FilePathField',
    'SlugField',
    'TypedChoiceField',
    'TypedMultipleChoiceField',
    'SplitDateTimeField',
    'ComboField',
    'MultiValueField',
    'NullBooleanField',
    'RegexField',
    'Form',
    'ModelForm',
    'ValidationError',
)


from django.shortcuts import render
from django.template import loader, Context, Template, RequestContext
from django.utils import six
from django.template.base import TextNode
from django.template.loader_tags import BlockNode, ExtendsNode, BLOCK_CONTEXT_KEY, BlockContext
from django_helpers import get_settings_val
from django_helpers.apps.static_manager import jQueryURL
from django_helpers.apps.static_manager.utils import generate_js_from_context

from django_helpers.helpers.views import render_json
from . import get_block_id


__author__ = 'ajumell'

title_key = 'title'


def find_extends_node(nodelist):
    for node in nodelist:
        if not isinstance(node, TextNode):
            if isinstance(node, ExtendsNode):
                return node
                # ExtendsNode has to be the first node!
            return


def resolve_template(template):
    if isinstance(template, Template):
        return template

    elif isinstance(template, (list, tuple)):
        return loader.select_template(template)

    elif isinstance(template, six.string_types):
        return loader.get_template(template)

    else:
        return Template(template)


def resolve_context(context, request=None):
    if isinstance(context, Context):
        return context
    if isinstance(context, dict) or context is None:
        if request is not None:
            return RequestContext(request, context)
        else:
            return Context(context)
    else:
        raise


# noinspection PyUnresolvedReferences
def is_partial_request(request):
    """

    @param request: django.http.HttpRequest
    @return: bool
    """
    is_ajax = request.META.get('HTTP_X_DJANGO_AJAX_RENDERER', None) == 'true'
    blocks = request.GET.getlist('__blocks__[]', [])
    return blocks if is_ajax else None


def get_app_name(request):
    app_name = request.GET.get('__app_name__', "")
    return None if app_name == '' else app_name


def ajax_render(request, template, context=None):
    blocks = is_partial_request(request)
    app_name = get_app_name(request)
    if blocks is None:
        return render(request, template, context)

    template = resolve_template(template)
    context = resolve_context(context, request)
    title = ''
    if title_key in context:
        title = context[title_key]

    blocks = render_blocks(template, context, blocks)

    if title_key in blocks:
        title = blocks[title_key]

    op = generate_js_from_context(context, app_name)
    static_url = get_settings_val('STATIC_URL')
    for block in op:
        files = block['files']
        url = static_url + jQueryURL
        while url in files:
            files.remove(url)

    return render_json({
        'blocks': blocks,
        'url': request.get_full_path(),
        'js': op,
        'title': title
    })


def render_blocks(template, context, ajax_blocks):
    """
    @param template: A Template object
    @param context: A Context or RequestContext object
    @param ajax_blocks: A list of blocks
    @return: dict of blocks and its data.

    Render the requested blocks into a dictionary.
    """
    templates = [template]

    # Find all templates in the inheritance chain.
    while template:
        extends_node = find_extends_node(template.nodelist)
        if extends_node:
            template = extends_node.get_parent(context)
            templates.append(template)
        else:
            break

    # Ensure we have the proper BlockContext.
    if not BLOCK_CONTEXT_KEY in context.render_context:
        context.render_context[BLOCK_CONTEXT_KEY] = BlockContext()
    block_context = context.render_context[BLOCK_CONTEXT_KEY]

    # Collect all the BlockNodes and fill the structure.
    for template in templates:
        extends_node = find_extends_node(template.nodelist)
        if extends_node:
            block_context.add_blocks(extends_node.blocks)
        else:  # Root template
            blocks = dict([(n.name, n) for n in template.nodelist.get_nodes_by_type(BlockNode)])
            block_context.add_blocks(blocks)
    res = {}
    for name in ajax_blocks:
        block = block_context.get_block(name)
        res[get_block_id(name)] = block.render(context)

    if title_key not in ajax_blocks:
        block = block_context.get_block(title_key)
        if block is not None:
            res[title_key] = block.render(context)
    return res


# coding=utf-8
from django import forms
from django_helpers.apps.form_renderer import FormRenderer
from models import Image

__author__ = 'ajumell'


class AddImageForm(forms.ModelForm):
    class Meta:
        model = Image

class EditImageForm(forms.ModelForm):
    class Meta:
        model = Image
        exclude = ('image', )


class AddImageFormRenderer(FormRenderer):
    form_id = 'add-image-form'
    form_submit = 'Upload Image'



# coding=utf-8
"""

Copied from

http://code.google.com/p/minidetector/
All credits to original author.

"""
from django_helpers import get_settings_val
from django_helpers.helpers.views import redirect

from useragents import search_strings


class Middleware(object):
    def process_request(self, request):
        """Adds a "mobile" attribute to the request which is True or False
           depending on whether the request should be considered to come from a
           small-screen device such as a phone or a PDA"""

        MOBILE_URL = get_settings_val('REDIRECT_MOBILE', None)

        if request.META.has_key("HTTP_X_OPERAMINI_FEATURES"):
            #Then it's running opera mini. 'Nuff said.
            #Reference from:
            # http://dev.opera.com/articles/view/opera-mini-request-headers/
            request.mobile = True
            return None

        if request.META.has_key("HTTP_ACCEPT"):
            s = request.META["HTTP_ACCEPT"].lower()
            if 'application/vnd.wap.xhtml+xml' in s:
                if MOBILE_URL is None:
                    # Then it's a wap browser
                    request.mobile = True
                    return None
                else:
                    return redirect(MOBILE_URL)

        if request.META.has_key("HTTP_USER_AGENT"):
            # This takes the most processing. Surprisingly enough, when I
            # Experimented on my own machine, this was the most efficient
            # algorithm. Certainly more so than regexes.
            # Also, Caching didn't help much, with real-world caches.
            s = request.META["HTTP_USER_AGENT"].lower()
            for ua in search_strings:
                if ua in s:
                    if MOBILE_URL is None:
                        request.mobile = True
                        return None
                    else:
                        return redirect(MOBILE_URL)

        #Otherwise it's not a mobile
        request.mobile = False
        return None


def detect_mobile(view):
    """View Decorator that adds a "mobile" attribute to the request which is
       True or False depending on whether the request should be considered
       to come from a small-screen device such as a phone or a PDA"""

    def detected(request, *args, **kwargs):
        obj = Middleware()
        obj.process_request(request)
        return view(request, *args, **kwargs)

    detected.__doc__ = "%s\n[Wrapped by detect_mobile which detects if the request is from a phone]" % view.__doc__
    return detected


__all__ = ['Middleware', 'detect_mobile']


# coding=utf-8
from django import template
from django.template.base import kwarg_re, TemplateSyntaxError, Node

__author__ = 'ajumell'


class TemplateNode(Node):
    def __init__(self, args=None, kwargs=None):
        self.args = args
        self.kwarg = kwargs


def parse_args(bits, parser):
    """

    @param bits: The token bits.
    @param parser: The template parser give to tag.
    @return: arguments and keyword arguments.
    """
    args = []
    kwargs = {}
    for bit in bits[1:]:
        match = kwarg_re.match(bit)
        if not match:
            raise TemplateSyntaxError("Malformed arguments to %s tag" % bits[0])
        name, value = match.groups()
        val = parser.compile_filter(value)
        if name:
            kwargs[name] = val
        else:
            if len(kwargs) == 0:
                args.append(val)
            else:
                raise TemplateSyntaxError("A normal argument cannot preceed keyword argument.")
    return args, kwargs


def resolve_args(args, kwargs, context):
    new_args = []
    new_kwargs = {}

    if args is not None:
        for arg in args:
            new_args.append(arg.resolve(context))

    if kwargs is not None:
        for kwarg in kwargs:
            new_kwargs[kwarg] = kwargs[kwarg].resolve(context)

    if kwargs is not None and args is not None:
        return new_args, new_kwargs

    if kwargs is not None:
        return new_kwargs

    if args is not None:
        return new_args


def create_context_from_kwargs(kwargs, new_context, names, context):
    for name in names:
        if kwargs.has_key(name):
            new_context[name] = kwargs[name].resolve(context)


def parse_verbatim(parser, endtag):
    text = []
    while 1:
        token = parser.tokens.pop(0)
        if token.contents == endtag:
            break
        if token.token_type == template.TOKEN_VAR:
            text.append('{{ ')
        elif token.token_type == template.TOKEN_BLOCK:
            text.append('{% ')
        text.append(token.contents)
        if token.token_type == template.TOKEN_VAR:
            text.append(' }}')
        elif token.token_type == template.TOKEN_BLOCK:
            text.append(' %}')
    return ''.join(text)


# coding=utf-8
from django.template import Template, loader, RequestContext

__author__ = 'ajumell'

_lookups = (
    "exact",
    "iexact",
    "contains",
    "icontains",
    "in",
    "gt",
    "gte",
    "lt",
    "lte",
    "startswith",
    "istartswith",
    "endswith",
    "iendswith",
    "range",
    "year",
    "month",
    "day",
    "week_day",
    "isnull",
    "search",
    "regex",
    "iregex"
)


def get_value(instance, field, format_expression=None):
    try:
        value = getattr(instance, field)
    except Exception:
        if field.find('__') > 0:
            fields = field.split('__')
        elif field.find('.') > 0:
            fields = field.split('.')
        else:
            raise
        value = instance
        for field in fields:
            value = getattr(value, field)
    if format_expression is not None:
        value = format_expression % value

    return value


class DataTableColumn(object):
    def __init__(self, field, title, searchable=False, sortable=False, title_class=None, title_width=None,
                 editable=False, annotate=None, search_lookup='contains', format_expression=None):
        self.field = field
        self.title = title
        self.searchable = searchable
        self.sortable = sortable
        self.title_class = title_class
        self.title_width = title_width
        self.editable = editable
        self.format_expression = format_expression

        if searchable and search_lookup not in _lookups:
            raise

        self.search_lookup = search_lookup

        if annotate is not None:
            from data_table import validate_annotate

            if not validate_annotate(annotate):
                raise

            if editable:
                raise

            self.field = field + "__" + annotate

        self.annotate = annotate

    def get_html(self, request, instance):
        return get_value(instance, self.field, self.format_expression)

    def need_related(self):
        return self.annotate is None and self.field.find('__') > -1


class DataTableClientRenderedColumn(DataTableColumn):
    """
    This class is still under planning.
    """
    pass


class DataTableTemplateColumn(DataTableColumn):
    """
    A Data table column which will return a rendered
    a template instead of a database field.
    """

    def __init__(self, field, title, searchable, sortable, template,
                 load_template=True, width=None, value_name='value', title_class=None, title_width=None, editable=False, annotate=None, search_lookup='contains', format_expression=None):
        """
        @param field: The name of the field to be passed to the template.
        @param title: Title of the column which will be displayed in the table header.
        @param searchable: jQuery data table option searchable
        @param sortable: jQuery data table option sortable
        @param template: Template to be rendered as the return value.
        This can be either a django.template.Template object or a sting.
        If a string is passed then that template has to be loaded by give
        load_template parameter as True.
        @param load_template: If this parameter is true the template will
        be loaded.
        @param width: Width of the column
        @param value_name: name of the variable that has to be passed to the template.
        Its default value is "value"
        """
        DataTableColumn.__init__(self, field, title, searchable, sortable, title_class, title_width, editable, annotate, search_lookup, format_expression)
        if load_template:
            template = loader.get_template(template)
        self.template = template
        self.width = width
        self.value_name = value_name

    def get_html(self, request, instance):
        value = getattr(instance, self.field)
        template = self.template

        if not isinstance(template, Template):
            template = Template(self.template)

        context = RequestContext(request, {
            self.value_name: value,
            "width": self.width
        })

        return template.render(context)



# coding=utf-8
def make_class_names(d):
    if isinstance(d, dict):
        class_names = d.pop('class', '')
    else:
        class_names = d

    if type(class_names) in (tuple, list):
        class_names = ' '.join(class_names)
    return class_names.strip()


def combine_class_names(one, two):
    name = "%s %s" % (one, two)
    return name.strip()


def extra_widget_args(renderer):
    form = renderer.form
    datas = getattr(renderer, 'widget_attrs', None)
    if type(datas) is not dict:
        return
    for name, data in datas.items():
        field = form.fields[name]
        widget = field.widget

        class_names = make_class_names(data)
        previous_classes = widget.attrs.pop('class', '')
        updated_class_names = combine_class_names(class_names, previous_classes)

        if updated_class_names != '':
            data['class'] = updated_class_names

        widget.attrs.update(data)


def extra_widget_class_names(renderer):
    form = renderer.form
    datas = getattr(renderer, 'class_names', None)
    if type(datas) is not dict:
        return

    for name, data in datas.items():
        field = form.fields[name]
        widget = field.widget

        class_names = make_class_names(data)
        previous_classes = widget.attrs.pop('class', '')
        updated_class_names = combine_class_names(class_names, previous_classes)

        if updated_class_names != '':
            widget.attrs['class'] = updated_class_names


from django_helpers.apps.static_manager import jQueryURL
from django_helpers.helpers.views import render_to_string

__author__ = 'ajumell'

import os

from django import template

from django.contrib.staticfiles import finders

from django_helpers import get_settings_val

from ..exceptions import AppNameMissingException
from ..storage import get_files
from ..utils import add_js_from_context, resolve_file_name, generate_js_from_context

STATIC_URL = get_settings_val('STATIC_URL')
_IS_DEBUG = get_settings_val('DEBUG', True)
SAVE_FILE = get_settings_val('STATIC_MANAGER_MAKE_FILE', True)
register = template.Library()


def get_path(path):
    dirs = get_settings_val('STATICFILES_DIRS')
    if len(dirs) is 0:
        return

    root = dirs[0]
    root = os.path.abspath(root)

    if not os.path.exists(root):
        raise Exception('%s does not exists.' % root)

    new_path = os.path.join(root, path)
    if os.path.exists(new_path):
        return new_path

    new_path = root
    parts = os.path.split(path)
    name = parts[-1]
    parts = parts[:-1]
    for part in parts:
        new_path = os.path.join(new_path, part)
        if os.path.exists(new_path):
            if not os.path.isdir(new_path):
                raise
        else:
            os.mkdir(new_path)
    new_path = os.path.join(new_path, name)
    return new_path


def save_file(path, array):
    if not SAVE_FILE:
        return

    path = get_path(path)
    if path is None:
        return

    contents = ''
    for file_name in array:
        file_name = resolve_file_name(file_name)
        real = finders.find(file_name)
        if real is None:
            raise Exception('File %s not found.' % file_name)
        fp = file(real)
        contents += fp.read()
        fp.close()
    fp = file(path, 'w')
    fp.write(contents)
    fp.close()


def _render_code(files, template):
    code = []
    for f in files:
        f = unicode(f)
        if f.startswith("http://") or f.startswith("https://"):
            s = ""
        else:
            s = STATIC_URL
        code.append(template % (s, f))
    return '\n'.join(code)


def has_ajax_rendering(context):
    return 'ajax_block_register' in context


def resolve_app_name(context, app_name):
    if app_name is None:
        if 'app_name' not in context:
            raise AppNameMissingException()
        else:
            return context['app_name']
    elif app_name in ['js', 'css', 'all']:
        raise
    return app_name


@register.simple_tag(takes_context=True)
def static_files(context, app_name=None, file_type='js'):
    if file_type == 'js':
        return js_files(context, app_name)
    elif file_type == 'css':
        return css_files(context, app_name)
    else:
        raise


@register.simple_tag(takes_context=True)
def render_js(context, app_name=None):
    op = generate_js_from_context(context, app_name)
    if has_ajax_rendering(context):
        static_url = get_settings_val('STATIC_URL')
        if app_name is None:
            raise

        for block in op:
            files = block['files']
            url = static_url + jQueryURL
            while url in files:
                files.remove(url)

        generated_code = render_to_string('static-manager/render-js.html', {
            'code_blocks': op,
            'STATIC_URL': static_url,
            'app_name': app_name
        })
    else:
        generated_code = '<script type="text/javascript">'
        for block in op:
            generated_code += block['js_code']
    return generated_code + '</script>'


@register.simple_tag(takes_context=True)
def js_files(context, app_name=None):
    app_name = resolve_app_name(context, app_name)
    add_js_from_context(context, app_name)
    if has_ajax_rendering(context):
        return render_js(context, app_name)
    else:
        files = get_files('js', app_name)
        template = '<script type="text/javascript" src="%s%s"></script>'
        return _render_code(files, template) + render_js(context, app_name)


@register.simple_tag(takes_context=True)
def css_files(context, app_name=None):
    app_name = resolve_app_name(context, app_name)
    files = get_files('css', app_name)
    template = '<link rel="stylesheet" href="%s%s" />'
    return _render_code(files, template)



# coding=utf-8
from django.db import models
from django_helpers.apps.image_manager.fields import ImageManagerImageField
from django_helpers.apps.nivo_slider import EffectsChoices, ThemesChoices


__author__ = 'ajumell'

class NivoSlider(models.Model):
    name = models.CharField(max_length=32, unique=True)
    effect = models.CharField(max_length=75, choices=EffectsChoices)
    slices = models.SmallIntegerField(default=15)

    box_cols = models.SmallIntegerField(default=8)
    box_rows = models.SmallIntegerField(default=4)

    animation_speed = models.SmallIntegerField(default=700)
    pause_time = models.SmallIntegerField(default=3000)

    direction_nav = models.BooleanField(default=True)
    control_nav = models.BooleanField(default=True)
    control_bav_thumbs = models.BooleanField(default=False)
    pause_on_hover = models.BooleanField(default=True)
    manual_advance = models.BooleanField(default=False)

    prev_text = models.CharField(max_length=32, default='Prev')
    next_text = models.CharField(max_length=32, default='Next')

    random_start = models.BooleanField(default=False)
    theme_name = models.CharField(max_length=75, default='bar', choices=ThemesChoices)

    width = models.IntegerField(null=True, blank=True)
    height = models.IntegerField(null=True, blank=True)


    def slides(self):
        return self.nivosliderslide_set.all()


    def __unicode__(self):
        return self.name


class NivoSliderSlide(models.Model):
    slider = models.ForeignKey(NivoSlider)
    image = ImageManagerImageField()
    link = models.URLField(blank=True)
    caption = models.CharField(max_length=255, blank=True)
    transition = models.CharField(max_length=75, choices=EffectsChoices, blank=True)

    @property
    def alternate_text(self):
        return self.image.alternate_text

    def __unicode__(self):
        return "%d - %s" % (self.id, self.image.name)


from types import StringTypes

from django.contrib.staticfiles import finders

from django_helpers import get_settings_val
from exceptions import PYESPRIMAException, SyntaxErrorException
from storage import add_js_files
from exceptions import StaticFileNotFoundException


INCLUDE_CSS = get_settings_val('STATIC_MANAGER_INCLUDE_CSS', True)
INCLUDE_JS = get_settings_val('STATIC_MANAGER_INCLUDE_JS', True)


def file_exists(name):
    if finders.find(name) is None:
        if get_settings_val('RAISE_STATIC_EXCEPTIONS', True):
            raise StaticFileNotFoundException(name)
        else:
            print StaticFileNotFoundException(name)
            return False
    else:
        return True


def resolve_file_name(file_name):
    t = type(file_name)
    if t in (list, tuple):
        if len(file_name) is 1:
            return file_name[0]
        else:
            raise
    if t in StringTypes:
        return file_name

    raise


def remove_duplicates(arr):
    """
    @summary Removes duplicate entries in the list and
    append the static dir prefix if add static is True.

    @author Muhammed K K
    @rtype list


    @param arr: The list from which duplicates has to be removed.
    @return: The newly created array with duplicates removed.
    """
    new_arr = []
    for js in arr:
        if js not in new_arr:
            new_arr.append(js)
    return new_arr


def has_js(obj):
    """
    @summary Checks whether a object has javascript in it or not.
    @param obj: The object from which has javascript to be checked.
    @return: yes or no
    """
    has_meta = hasattr(obj, '_meta')
    _has_js = hasattr(obj, 'render_js') and getattr(obj, 'has_js', False)

    if not has_meta:
        return _has_js

    _meta = obj._meta
    _has_js_meta = hasattr(obj, 'render_js') and getattr(_meta, 'has_js', False)

    return _has_js or _has_js_meta


def get_js_files(obj):
    """
    @param obj: The object from which the list of javascript files has to be found.
    @return: A list of required javascript file.
    """
    if not has_js(obj):
        return []

    if not hasattr(obj, 'js_files'):
        return []

    files = obj.js_files

    if hasattr(files, '__call__'):
        files = obj.js_files()

    if type(files) not in (list, tuple):
        raise Exception('js_files is not in correct format in %s.' % str(type(obj)))

    files = remove_duplicates(files)
    return files


# noinspection PyUnresolvedReferences
def validate_javascript(code):
    try:
        import pyesprima
    except ImportError:
        raise PYESPRIMAException()
    try:
        pyesprima.parse(code)
        return True
    except Exception:
        return False


def load_js_files_from_context(context):
    js_arr = []
    for dictionary in context.dicts:
        for key, val in dictionary.items():
            js_arr += get_js_files(val)
    return js_arr


def add_js_from_context(context, app_name):
    js_arr = load_js_files_from_context(context)
    add_js_files(app_name, js_arr)


def append_static_arr(arr):
    static_url = get_settings_val('STATIC_URL')
    return [static_url + a for a in arr]


def generate_js_from_context(context, app_name=None):
    js_arr = []
    for dictionary in context.dicts:
        for key, val in dictionary.items():
            if has_js(val):
                js_files = get_js_files(val)

                if app_name is not None:
                    add_js_files(app_name, js_files)

                js_files = append_static_arr(js_files)
                js_code = val.render_js().strip()

                if not validate_javascript(js_code):
                    raise SyntaxErrorException(val)

                js_arr.append({
                    'files': js_files,
                    'js_code': js_code
                })
    return js_arr


# coding=utf-8
from django import template
from django_helpers.utils import models

register = template.Library()


@register.simple_tag(takes_context=False)
def last_query():
    return models.last_query()





from django.template import Node, TemplateSyntaxError, Library
from .. import get_menu
from django_helpers.helpers.templatetags import parse_args


__author__ = 'ajumell'

register = Library()


class MenuRenderNode(Node):
    def __init__(self, menu_name, args=None, kwargs=None):
        self.menu_name = menu_name
        self.args = args
        self.kwargs = kwargs

    def render(self, context):
        menu_name = self.menu_name.resolve(context)
        menu_cls = get_menu(menu_name)
        menu = menu_cls(context)
        context[menu.menu_id] = menu
        return menu.render()


@register.tag
def render_menu(parser, token):
    bits = token.split_contents()
    if len(bits) < 2:
        raise TemplateSyntaxError("'%s' needs at least two arguments." % bits[0])
    args, kwargs = parse_args(bits, parser)

    table_name = args[0]
    args = args[1:]

    return MenuRenderNode(table_name, args, kwargs)


from django.conf.urls import url
from django.http import HttpResponse
from django_helpers import add_app_url

__author__ = 'ajumell'
import urls
from editable import XEditable

_URL_PREFIX = 'x-editable-'

_EDITABLES = dict()


def create_reg(name):
    return "(?P<%s>.*)/" % name


def get_editable(name, pk):
    if name not in _EDITABLES:
        return None

    return _EDITABLES[name](pk=pk)


def save(request, name):
    if request.method != 'POST':
        return HttpResponse()

    gt = request.POST.get
    pk = gt('pk')
    editable = get_editable(name, pk)
    if editable is None:
        return HttpResponse()
    field_name = gt('name')
    results = editable.save(request, field_name)
    return results


def list_data(request, name, field_name):
    gt = request.POST.get
    pk = gt('pk')
    editable = get_editable(name, pk)
    if editable is None:
        return HttpResponse()
    results = editable.list(field_name)
    return results


def register(editable):
    add_app_url('data-table', urls)

    if isinstance(editable, XEditable):
        raise Exception("XEditable class is required not instance.")

    if not issubclass(editable, XEditable):
        raise Exception("A Sub class of XEditable is required.")

    name = getattr(editable, 'editable_id', None)
    if name is None:
        raise Exception("Editable ID is needed.")

    if not name.startswith(_URL_PREFIX):
        name = _URL_PREFIX + name

    if name in _EDITABLES:
        old_editable = _EDITABLES[name]

        new_name = str(editable)
        old_name = str(old_editable)

        if old_name == new_name:
            return editable

        msg = "You tried to register a editable with id " + name + " for " + new_name + ".\n"
        msg += "But " + name + " is already registered for " + old_name + ".\n"
        msg += "Try giving a new ID for " + new_name
        raise Exception(msg)

    setattr(editable, 'editable_id', name)
    _EDITABLES[name] = editable

    list_reg = "%s/" % name
    list_reg += create_reg('field_name')
    list_reg += "list/$"

    save_reg = "%s/" % name
    save_reg += "save/$"

    save_pattern = url(r"%s" % save_reg, save, name=name + '_save', kwargs={
        "name": name
    })

    list_pattern = url(r"%s" % list_reg, list_data, name=name + '_list', kwargs={
        "name": name
    })

    urls.urlpatterns.append(save_pattern)
    urls.urlpatterns.append(list_pattern)
    return editable


# coding=utf-8
from django.forms import Field, Widget


def replace_widgets(renderer):
    form = renderer.form
    datas = getattr(renderer, 'replace_widgets', None)
    if type(datas) is not dict:
        return

    if len(datas) == 0:
        return

    for name, field in form.fields.items():
        if isinstance(field, Field):
            old_widget = field.widget
            if isinstance(old_widget, Widget):
                widget_cls = type(field.widget)
                new_cls = datas.get(widget_cls)
                if new_cls is None:
                    continue
                if isinstance(new_cls, type):
                    new_cls = new_cls()
                if isinstance(new_cls, Widget):
                    new_cls.is_localized = old_widget.is_localized
                    new_cls.is_required = old_widget.is_required
                    field.widget = new_cls